@@ -24,6 +24,7 @@
 #define release_wake_lock(id) {}
 
 #include "mir/input/input_report.h"
+#include "mir/udev/wrapper.h"
 
 #include <cutils/properties.h>
 #include <std/Log.h>
@@ -212,7 +213,7 @@ const int EventHub::EPOLL_MAX_EVENTS;
 
 EventHub::EventHub(std::shared_ptr<mi::InputReport> const& input_report) :
         input_report(input_report),
-        device_listener{mir::udev::Context()},
+        device_listener{new mir::udev::Monitor{mir::udev::Context()}},
         mBuiltInKeyboardId(NO_BUILT_IN_KEYBOARD), mNextDeviceId(1),
         mOpeningDevices(0), mClosingDevices(0),
         mNeedToSendFinishedDeviceScan(false),
@@ -223,15 +224,15 @@ EventHub::EventHub(std::shared_ptr<mi::InputReport> const& input_report) :
     mEpollFd = epoll_create(EPOLL_SIZE_HINT);
     LOG_ALWAYS_FATAL_IF(mEpollFd < 0, "Could not create epoll instance.  errno=%d", errno);
 
-    device_listener.filter_by_subsystem("input");
-    device_listener.enable();
+    device_listener->filter_by_subsystem("input");
+    device_listener->enable();
 
 
     struct epoll_event eventItem;
     memset(&eventItem, 0, sizeof(eventItem));
     eventItem.events = EPOLLIN;
     eventItem.data.u32 = EPOLL_ID_UDEV;
-    int result = epoll_ctl(mEpollFd, EPOLL_CTL_ADD, device_listener.fd(), &eventItem);
+    int result = epoll_ctl(mEpollFd, EPOLL_CTL_ADD, device_listener->fd(), &eventItem);
     LOG_ALWAYS_FATAL_IF(result != 0, "Could not add Udev monitor to epoll instance.  errno=%d", errno);
 
     int wakeFds[2];
@@ -579,7 +580,7 @@ bool EventHub::setKeyboardLayoutOverlay(int32_t deviceId,
     return false;
 }
 
-void EventHub::vibrate(int32_t deviceId, nsecs_t duration) {
+void EventHub::vibrate(int32_t deviceId, std::chrono::nanoseconds duration) {
     AutoMutex _l(mLock);
     Device* device = getDeviceLocked(deviceId);
     if (device && !device->isVirtual()) {
@@ -589,7 +590,7 @@ void EventHub::vibrate(int32_t deviceId, nsecs_t duration) {
         effect.id = device->ffEffectId;
         effect.u.rumble.strong_magnitude = 0xc000;
         effect.u.rumble.weak_magnitude = 0xc000;
-        effect.replay.length = (duration + 999999LL) / 1000000LL;
+        effect.replay.length = (duration + std::chrono::nanoseconds(999999)) / std::chrono::nanoseconds(1000000LL);
         effect.replay.delay = 0;
         if (ioctl(device->fd, EVIOCSFF, &effect)) {
             ALOGW("Could not upload force feedback effect to device %s due to error %d.",
@@ -664,7 +665,7 @@ size_t EventHub::getEvents(int timeoutMillis, RawEvent* buffer, size_t bufferSiz
     size_t capacity = bufferSize;
     bool awoken = false;
     for (;;) {
-        nsecs_t now = systemTime(SYSTEM_TIME_MONOTONIC);
+        std::chrono::nanoseconds now = systemTime(SYSTEM_TIME_MONOTONIC);
 
         // Reopen input devices if needed.
         if (mNeedToReopenDevices) {
@@ -801,8 +802,8 @@ size_t EventHub::getEvents(int timeoutMillis, RawEvent* buffer, size_t bufferSiz
                         // The systemTime(SYSTEM_TIME_MONOTONIC) function we use everywhere
                         // calls clock_gettime(CLOCK_MONOTONIC) which is implemented as a
                         // system call that also queries ktime_get_ts().
-                        event->when = nsecs_t(iev.time.tv_sec) * 1000000000LL
-                                + nsecs_t(iev.time.tv_usec) * 1000LL;
+                        event->when = std::chrono::nanoseconds(iev.time.tv_sec) * 1000000000LL
+                                + std::chrono::nanoseconds(iev.time.tv_usec) * 1000LL;
                         ALOGV("event time %lld, now %lld", event->when, now);
 #else
                         event->when = now;
@@ -812,7 +813,7 @@ size_t EventHub::getEvents(int timeoutMillis, RawEvent* buffer, size_t bufferSiz
                         event->code = iev.code;
                         event->value = iev.value;
 
-                        input_report->received_event_from_kernel(event->when, event->type, event->code, event->value);
+                        input_report->received_event_from_kernel(event->when.count(), event->type, event->code, event->value);
 
                         event += 1;
                     }
@@ -935,7 +936,7 @@ void EventHub::scanDevicesLocked() {
 
 void EventHub::handleUdevEventsLocked()
 {
-    device_listener.process_events([this](mir::udev::Monitor::EventType type, mir::udev::Device const& dev){
+    device_listener->process_events([this](mir::udev::Monitor::EventType type, mir::udev::Device const& dev){
         if (type == mir::udev::Monitor::ADDED)
         {
             if (dev.devnode() != nullptr)
