@@ -20,6 +20,7 @@
 #define MIR_COMPOSITOR_BUFFER_H_
 
 #include "mir/graphics/native_buffer.h"
+#include "mir/graphics/buffer_id.h"
 #include "mir/geometry/size.h"
 #include "mir_toolkit/common.h"
 
@@ -29,7 +30,6 @@ namespace mir
 {
 namespace graphics
 {
-class BufferID;
 
 class Buffer
 {
@@ -41,8 +41,11 @@ public:
     virtual geometry::Size size() const = 0;
     virtual geometry::Stride stride() const = 0;
     virtual MirPixelFormat pixel_format() const = 0;
-    virtual void bind_to_texture() = 0;
+    virtual void gl_bind_to_texture() = 0;
+    /* TODO: remove this function, as it is specific to the mesa platform */
     virtual bool can_bypass() const = 0;
+    
+    virtual void write(unsigned char const* pixels, size_t size) = 0;
 
 protected:
     Buffer() = default;
