@@ -22,18 +22,11 @@
 #include "mir_toolkit/common.h"
 #include "mir/frontend/surface_id.h"
 
-#include <mutex>
-#include <atomic>
 #include <memory>
 #include <string>
 
 namespace mir
 {
-namespace shell
-{
-struct SurfaceCreationParameters;
-}
-
 namespace graphics
 {
 class DisplayConfiguration;
@@ -46,20 +39,12 @@ class Surface;
 class Session
 {
 public:
-    virtual ~Session() {}
+    virtual ~Session() = default;
 
-    virtual SurfaceId create_surface(shell::SurfaceCreationParameters const& params) = 0;
-    virtual void destroy_surface(SurfaceId surface) = 0;
     virtual std::shared_ptr<Surface> get_surface(SurfaceId surface) const = 0;
 
     virtual std::string name() const = 0;
 
-    virtual void hide() = 0;
-    virtual void show() = 0;
-
-    virtual void send_display_config(graphics::DisplayConfiguration const&) = 0;
-    virtual int configure_surface(SurfaceId id, MirSurfaceAttrib attrib, int value) = 0;
-
 protected:
     Session() = default;
     Session(Session const&) = delete;
