@@ -39,8 +39,6 @@ namespace logging
 class Logger;
 }
 
-class SharedLibrary;
-
 namespace client
 {
 class ConnectionSurfaceMap;
@@ -65,7 +63,6 @@ public:
     virtual std::shared_ptr<LifecycleControl> the_lifecycle_control() = 0;
     virtual std::shared_ptr<EventSink> the_event_sink() = 0;
     virtual std::shared_ptr<EventHandlerRegister> the_event_handler_register() = 0;
-    virtual std::shared_ptr<SharedLibrary> the_platform_library() = 0;
 
 protected:
     ConnectionConfiguration() = default;
