@@ -18,24 +18,29 @@
 
 #include "mir_toolkit/mir_client_library.h"
 #include "mir/frontend/client_constants.h"
-#include "client_buffer.h"
+#include "mir/client_buffer.h"
 #include "mir_surface.h"
 #include "cursor_configuration.h"
+#include "client_buffer_stream_factory.h"
 #include "mir_connection.h"
 #include "mir/input/input_receiver_thread.h"
 #include "mir/input/input_platform.h"
-#include "perf_report.h"
-#include "logging/perf_report.h"
 
 #include <cassert>
 #include <unistd.h>
 
+#include <boost/exception/diagnostic_information.hpp>
+
 namespace geom = mir::geometry;
 namespace mcl = mir::client;
 namespace mircv = mir::input::receiver;
 namespace mp = mir::protobuf;
 namespace gp = google::protobuf;
 
+#define SERIALIZE_OPTION_IF_SET(option, message) \
+    if (option.is_set()) \
+        message.set_##option(option.value());
+
 namespace
 {
 void null_callback(MirSurface*, void*) {}
@@ -44,43 +49,93 @@ std::mutex handle_mutex;
 std::unordered_set<MirSurface*> valid_surfaces;
 }
 
+MirSurfaceSpec::MirSurfaceSpec(
+    MirConnection* connection, int width, int height, MirPixelFormat format)
+    : connection{connection},
+      width{width},
+      height{height},
+      pixel_format{format}
+{
+}
+
+MirSurfaceSpec::MirSurfaceSpec(MirConnection* connection, MirSurfaceParameters const& params)
+    : connection{connection},
+      width{params.width},
+      height{params.height},
+      pixel_format{params.pixel_format},
+      buffer_usage{params.buffer_usage}
+{
+    if (params.output_id != mir_display_output_id_invalid)
+    {
+        output_id = params.output_id;
+        state = mir_surface_state_fullscreen;
+    }
+}
+
+mir::protobuf::SurfaceParameters MirSurfaceSpec::serialize() const
+{
+    mir::protobuf::SurfaceParameters message;
+
+    message.set_width(width);
+    message.set_height(height);
+    message.set_pixel_format(pixel_format);
+    message.set_buffer_usage(buffer_usage);
+
+    SERIALIZE_OPTION_IF_SET(surface_name, message);
+    SERIALIZE_OPTION_IF_SET(output_id, message);
+    SERIALIZE_OPTION_IF_SET(type, message);
+    SERIALIZE_OPTION_IF_SET(state, message);
+    SERIALIZE_OPTION_IF_SET(pref_orientation, message);
+    if (parent.is_set() && parent.value() != nullptr)
+        message.set_parent_id(parent.value()->id());
+    if (aux_rect.is_set())
+    {
+        message.mutable_aux_rect()->set_left(aux_rect.value().left);
+        message.mutable_aux_rect()->set_top(aux_rect.value().top);
+        message.mutable_aux_rect()->set_width(aux_rect.value().width);
+        message.mutable_aux_rect()->set_height(aux_rect.value().height);
+    }
+    SERIALIZE_OPTION_IF_SET(edge_attachment, message);
+    return message;
+}
+
+MirSurface::MirSurface(std::string const& error)
+{
+    surface.set_error(error);
+
+    std::lock_guard<decltype(handle_mutex)> lock(handle_mutex);
+    valid_surfaces.insert(this);
+}
+
 MirSurface::MirSurface(
     MirConnection *allocating_connection,
-    mp::DisplayServer::Stub & server,
-    std::shared_ptr<mcl::ClientBufferFactory> const& factory,
+    mp::DisplayServer::Stub& the_server,
+    mp::Debug::Stub* debug,
+    std::shared_ptr<mcl::ClientBufferStreamFactory> const& buffer_stream_factory,
     std::shared_ptr<mircv::InputPlatform> const& input_platform,
-    MirSurfaceParameters const & params,
+    MirSurfaceSpec const& spec,
     mir_surface_callback callback, void * context)
-    : server(server),
+    : server{&the_server},
+      debug{debug},
+      name{spec.surface_name.value()},
       connection(allocating_connection),
-      buffer_depository(std::make_shared<mcl::ClientBufferDepository>(factory, mir::frontend::client_buffer_cache_size)),
+      buffer_stream_factory(buffer_stream_factory),
       input_platform(input_platform)
 {
-    const char* report_target = getenv("MIR_CLIENT_PERF_REPORT");
-    if (report_target && !strcmp(report_target, "log"))
-    {
-        auto& logger = connection->the_logger();
-        perf_report = std::make_shared<mir::client::logging::PerfReport>(logger);
-    }
-    else
-    {
-        perf_report = std::make_shared<mir::client::NullPerfReport>();
-    }
-    perf_report->name_surface(params.name);
-
     for (int i = 0; i < mir_surface_attribs; i++)
         attrib_cache[i] = -1;
 
-    mir::protobuf::SurfaceParameters message;
-    message.set_surface_name(params.name ? params.name : std::string());
-    message.set_width(params.width);
-    message.set_height(params.height);
-    message.set_pixel_format(params.pixel_format);
-    message.set_buffer_usage(params.buffer_usage);
-    message.set_output_id(params.output_id);
-
+    auto const message = spec.serialize();
     create_wait_handle.expect_result();
-    server.create_surface(0, &message, &surface, gp::NewCallback(this, &MirSurface::created, callback, context));
+    try 
+    {
+        server->create_surface(0, &message, &surface, gp::NewCallback(this, &MirSurface::created, callback, context));
+    }
+    catch (std::exception const& ex)
+    {
+        surface.set_error(std::string{"Error invoking create surface: "} +
+                          boost::diagnostic_information(ex));
+    }
 
     std::lock_guard<decltype(handle_mutex)> lock(handle_mutex);
     valid_surfaces.insert(this);
@@ -103,8 +158,6 @@ MirSurface::~MirSurface()
 
     for (auto i = 0, end = surface.fd_size(); i != end; ++i)
         close(surface.fd(i));
-
-    release_cpu_region();
 }
 
 MirSurfaceParameters MirSurface::get_parameters() const
@@ -152,9 +205,7 @@ void MirSurface::get_cpu_region(MirGraphicsRegion& region_out)
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
 
-    auto buffer = buffer_depository->current_buffer();
-
-    secured_region = buffer->secure_for_cpu_write();
+    auto secured_region = buffer_stream->secure_for_cpu_write();
     region_out.width = secured_region->width.as_uint32_t();
     region_out.height = secured_region->height.as_uint32_t();
     region_out.stride = secured_region->stride.as_uint32_t();
@@ -162,28 +213,14 @@ void MirSurface::get_cpu_region(MirGraphicsRegion& region_out)
     region_out.vaddr = secured_region->vaddr.get();
 }
 
-void MirSurface::release_cpu_region()
+MirWaitHandle* MirSurface::next_buffer(mir_surface_callback callback, void *context)
 {
-    secured_region.reset();
-}
-
-MirWaitHandle* MirSurface::next_buffer(mir_surface_callback callback, void * context)
-{
-    std::unique_lock<decltype(mutex)> lock(mutex);
-    release_cpu_region();
-    auto const id = &surface.id();
-    auto const mutable_buffer = surface.mutable_buffer();
-    perf_report->end_frame(mutable_buffer->buffer_id());
-    lock.unlock();
-
-    next_buffer_wait_handle.expect_result();
-    server.next_buffer(
-        0,
-        id,
-        mutable_buffer,
-        google::protobuf::NewCallback(this, &MirSurface::new_buffer, callback, context));
-
-    return &next_buffer_wait_handle;
+    return buffer_stream->next_buffer([&, callback, context]
+    {
+        process_incoming_buffer();
+        if (callback)
+            callback(this, context);
+    });
 }
 
 MirWaitHandle* MirSurface::get_create_wait_handle()
@@ -193,78 +230,66 @@ MirWaitHandle* MirSurface::get_create_wait_handle()
 
 /* todo: all these conversion functions are a bit of a kludge, probably
          better to have a more developed MirPixelFormat that can handle this */
-MirPixelFormat MirSurface::convert_ipc_pf_to_geometry(gp::int32 pf)
+MirPixelFormat MirSurface::convert_ipc_pf_to_geometry(gp::int32 pf) const
 {
     return static_cast<MirPixelFormat>(pf);
 }
 
 void MirSurface::process_incoming_buffer()
 {
-    auto const& buffer = surface.buffer();
+    auto const& buffer = get_current_buffer();
+    std::lock_guard<decltype(mutex)> lock(mutex);
 
-    /*
-     * On most frames when the properties aren't changing, the server won't
-     * fill in the width and height. I think this is an intentional
-     * protocol optimization ("need_full_ipc").
-     */
-    if (buffer.has_width() && buffer.has_height())
-    {
-        surface.set_width(buffer.width());
-        surface.set_height(buffer.height());
-    }
+    auto buffer_width = buffer->size().width.as_int();
+    auto buffer_height = buffer->size().height.as_int();
 
-    auto surface_size = geom::Size{surface.width(), surface.height()};
-    auto surface_pf = convert_ipc_pf_to_geometry(surface.pixel_format());
-
-    auto ipc_package = std::make_shared<MirBufferPackage>();
-    populate(*ipc_package);
-
-    try
-    {
-        buffer_depository->deposit_package(std::move(ipc_package),
-                                           buffer.buffer_id(),
-                                           surface_size, surface_pf);
-        perf_report->begin_frame(buffer.buffer_id());
-    }
-    catch (const std::runtime_error& err)
-    {
-        // TODO: Report the error
-    }
+    if (buffer_height != surface.width())
+        surface.set_width(buffer_width);
+    if (buffer_height != surface.height())
+        surface.set_width(buffer_height);
 }
 
 void MirSurface::created(mir_surface_callback callback, void * context)
 {
-    auto platform = connection->get_client_platform();
-
     {
-        std::lock_guard<decltype(mutex)> lock(mutex);
+    std::lock_guard<decltype(mutex)> lock(mutex);
+    if (!surface.has_id())
+    {
+        if (!surface.has_error())
+            surface.set_error("Error processing surface create response, no ID (disconnected?)");
 
-        process_incoming_buffer();
-        accelerated_window = platform->create_egl_native_window(this);
-        
-        for(int i = 0; i < surface.attributes_size(); i++)
+        callback(this, context);
+        create_wait_handle.result_received();
+        return;
+    }
+    }
+    try
+    {
         {
-            auto const& attrib = surface.attributes(i);
-            attrib_cache[attrib.attrib()] = attrib.ivalue();
+            std::lock_guard<decltype(mutex)> lock(mutex);
+
+            buffer_stream = buffer_stream_factory->
+                make_producer_stream(*server, surface.buffer_stream());
+
+            for(int i = 0; i < surface.attributes_size(); i++)
+            {
+                auto const& attrib = surface.attributes(i);
+                attrib_cache[attrib.attrib()] = attrib.ivalue();
+            }
         }
+
+        connection->on_surface_created(id(), this);
+    }
+    catch (std::exception const& error)
+    {
+        surface.set_error(std::string{"Error processing Surface creating response:"} +
+                          boost::diagnostic_information(error));
     }
 
-    connection->on_surface_created(id(), this);
     callback(this, context);
     create_wait_handle.result_received();
 }
 
-void MirSurface::new_buffer(mir_surface_callback callback, void * context)
-{
-    {
-        std::lock_guard<decltype(mutex)> lock(mutex);
-        process_incoming_buffer();
-    }
-
-    callback(this, context);
-    next_buffer_wait_handle.result_received();
-}
-
 MirWaitHandle* MirSurface::release_surface(
         mir_surface_callback callback,
         void * context)
@@ -274,7 +299,18 @@ MirWaitHandle* MirSurface::release_surface(
         valid_surfaces.erase(this);
     }
 
-    return connection->release_surface(this, callback, context);
+    MirWaitHandle* wait_handle{nullptr};
+    if (connection)
+    {
+        wait_handle = connection->release_surface(this, callback, context);
+    }
+    else
+    {
+        callback(this, context);
+        delete this;
+    }
+
+    return wait_handle;
 }
 
 MirNativeBuffer* MirSurface::get_current_buffer_package()
@@ -289,53 +325,21 @@ std::shared_ptr<mcl::ClientBuffer> MirSurface::get_current_buffer()
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
 
-    return buffer_depository->current_buffer();
+    return buffer_stream->get_current_buffer();
 }
 
 uint32_t MirSurface::get_current_buffer_id() const
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
 
-    return buffer_depository->current_buffer_id();
-}
-
-void MirSurface::populate(MirBufferPackage& buffer_package)
-{
-    if (!surface.has_error() && surface.has_buffer())
-    {
-        auto const& buffer = surface.buffer();
-
-        buffer_package.data_items = buffer.data_size();
-        for (int i = 0; i != buffer.data_size(); ++i)
-        {
-            buffer_package.data[i] = buffer.data(i);
-        }
-
-        buffer_package.fd_items = buffer.fd_size();
-
-        for (int i = 0; i != buffer.fd_size(); ++i)
-        {
-            buffer_package.fd[i] = buffer.fd(i);
-        }
-
-        buffer_package.stride = buffer.stride();
-        buffer_package.flags = buffer.flags();
-        buffer_package.width = buffer.width();
-        buffer_package.height = buffer.height();
-    }
-    else
-    {
-        buffer_package.data_items = 0;
-        buffer_package.fd_items = 0;
-        buffer_package.stride = 0;
-    }
+    return buffer_stream->get_current_buffer_id();
 }
 
 EGLNativeWindowType MirSurface::generate_native_window()
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
 
-    return *accelerated_window;
+    return buffer_stream->egl_native_window();
 }
 
 MirWaitHandle* MirSurface::configure_cursor(MirCursorConfiguration const* cursor)
@@ -350,7 +354,7 @@ MirWaitHandle* MirSurface::configure_cursor(MirCursorConfiguration const* cursor
     }
     
     configure_cursor_wait_handle.expect_result();
-    server.configure_cursor(0, &setting, &void_response,
+    server->configure_cursor(0, &setting, &void_response,
         google::protobuf::NewCallback(this, &MirSurface::on_cursor_configured));
     
     return &configure_cursor_wait_handle;
@@ -358,7 +362,18 @@ MirWaitHandle* MirSurface::configure_cursor(MirCursorConfiguration const* cursor
 
 MirWaitHandle* MirSurface::configure(MirSurfaceAttrib at, int value)
 {
+    // TODO: This is obviously strange. It should be
+    // possible to eliminate it in the second phase of buffer
+    // stream where the existing MirSurface swap interval functions
+    // may be deprecated in terms of mir_buffer_stream_ alternatives
+    if (at == mir_surface_attrib_swapinterval)
+    {
+        buffer_stream->set_swap_interval(value);
+        return &configure_wait_handle;
+    }
+
     std::unique_lock<decltype(mutex)> lock(mutex);
+
     mp::SurfaceSetting setting;
     setting.mutable_surfaceid()->CopyFrom(surface.id());
     setting.set_attrib(at);
@@ -366,12 +381,55 @@ MirWaitHandle* MirSurface::configure(MirSurfaceAttrib at, int value)
     lock.unlock();
 
     configure_wait_handle.expect_result();
-    server.configure_surface(0, &setting, &configure_result,
+    server->configure_surface(0, &setting, &configure_result,
               google::protobuf::NewCallback(this, &MirSurface::on_configured));
 
     return &configure_wait_handle;
 }
 
+namespace
+{
+void signal_response_received(MirWaitHandle* handle)
+{
+    handle->result_received();
+}
+}
+
+bool MirSurface::translate_to_screen_coordinates(int x, int y,
+                                                 int *screen_x, int *screen_y)
+{
+    if (!debug)
+    {
+        return false;
+    }
+
+    mp::CoordinateTranslationRequest request;
+
+    request.set_x(x);
+    request.set_y(y);
+    *request.mutable_surfaceid() = surface.id();
+    mp::CoordinateTranslationResponse response;
+
+    MirWaitHandle signal;
+    signal.expect_result();
+
+    {
+        std::lock_guard<decltype(mutex)> lock(mutex);
+
+        debug->translate_surface_to_screen(
+            nullptr,
+            &request,
+            &response,
+            google::protobuf::NewCallback(&signal_response_received, &signal));
+    }
+
+    signal.wait_for_one();
+
+    *screen_x = response.x();
+    *screen_y = response.y();
+    return !response.has_error();
+}
+
 void MirSurface::on_configured()
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
@@ -387,8 +445,8 @@ void MirSurface::on_configured()
         case mir_surface_attrib_type:
         case mir_surface_attrib_state:
         case mir_surface_attrib_focus:
-        case mir_surface_attrib_swapinterval:
         case mir_surface_attrib_dpi:
+        case mir_surface_attrib_preferred_orientation:
             if (configure_result.has_ivalue())
                 attrib_cache[a] = configure_result.ivalue();
             else
@@ -413,6 +471,14 @@ int MirSurface::attrib(MirSurfaceAttrib at) const
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
 
+    if (at == mir_surface_attrib_swapinterval)
+    {
+        if (buffer_stream)
+            return buffer_stream->swap_interval();
+        else // Surface creation is not finalized
+            return 1;
+    }
+
     return attrib_cache[at];
 }
 
@@ -446,19 +512,19 @@ void MirSurface::handle_event(MirEvent const& e)
 {
     std::unique_lock<decltype(mutex)> lock(mutex);
 
-    switch (e.type)
+    switch (mir_event_get_type(&e))
     {
     case mir_event_type_surface:
     {
-        MirSurfaceAttrib a = e.surface.attrib;
+        auto sev = mir_event_get_surface_event(&e);
+        auto a = mir_surface_event_get_attribute(sev);
         if (a < mir_surface_attribs)
-            attrib_cache[a] = e.surface.value;
+            attrib_cache[a] = mir_surface_event_get_attribute_value(sev);
         break;
     }
     case mir_event_type_orientation:
-        orientation = e.orientation.direction;
+        orientation = mir_orientation_event_get_direction(mir_event_get_orientation_event(&e));
         break;
-
     default:
         break;
     };
@@ -491,5 +557,12 @@ void MirSurface::request_and_wait_for_configure(MirSurfaceAttrib a, int value)
 
 MirOrientation MirSurface::get_orientation() const
 {
+    std::lock_guard<decltype(mutex)> lock(mutex);
+
     return orientation;
 }
+
+MirWaitHandle* MirSurface::set_preferred_orientation(MirOrientationMode mode)
+{
+    return configure(mir_surface_attrib_preferred_orientation, mode);
+}
