@@ -19,6 +19,9 @@
 #include "buffer_bundle.h"
 #include "temporary_buffers.h"
 
+#include <boost/throw_exception.hpp>
+#include <stdexcept>
+
 namespace mc=mir::compositor;
 namespace mg=mir::graphics;
 namespace geom=mir::geometry;
@@ -86,3 +89,9 @@ bool mc::TemporaryBuffer::can_bypass() const
 {
     return buffer->can_bypass();
 }
+
+void mc::TemporaryBuffer::write(unsigned char const*, size_t)
+{
+    BOOST_THROW_EXCEPTION(
+        std::runtime_error("Write to temporary buffer snapshot is ill advised and indicates programmer error"));
+}
