@@ -33,7 +33,7 @@
 
 namespace mir
 {
-namespace input { class EventFilter; }
+namespace input { class InputDispatcher; }
 namespace geometry
 {
 struct Rectangle;
@@ -44,6 +44,7 @@ class DisplayReport;
 class DisplayBuffer;
 class DisplayConfigurationPolicy;
 class GLConfig;
+class Platform;
 
 namespace nested
 {
@@ -66,13 +67,12 @@ private:
 class EGLDisplayHandle
 {
 public:
-    EGLDisplayHandle(MirConnection* connection,
+    EGLDisplayHandle(EGLNativeDisplayType native_display,
                      std::shared_ptr<GLConfig> const& gl_config);
     ~EGLDisplayHandle() noexcept;
 
     void initialize(MirPixelFormat format);
     EGLConfig choose_windowed_es_config(MirPixelFormat format) const;
-    EGLNativeWindowType native_window(EGLConfig egl_config, MirSurface* mir_surface) const;
     EGLContext egl_context() const;
     operator EGLDisplay() const { return egl_display; }
 
@@ -96,8 +96,9 @@ class NestedDisplay : public Display
 {
 public:
     NestedDisplay(
+        std::shared_ptr<Platform> const& platform,
         std::shared_ptr<HostConnection> const& connection,
-        std::shared_ptr<input::EventFilter> const& event_handler,
+        std::shared_ptr<input::InputDispatcher> const& dispatcher,
         std::shared_ptr<DisplayReport> const& display_report,
         std::shared_ptr<DisplayConfigurationPolicy> const& conf_policy,
         std::shared_ptr<GLConfig> const& gl_config);
@@ -121,18 +122,21 @@ public:
     void pause() override;
     void resume() override;
 
-    std::weak_ptr<Cursor> the_cursor() override;
+    std::shared_ptr<Cursor> create_hardware_cursor(std::shared_ptr<CursorImage> const& initial_image) override;
     std::unique_ptr<graphics::GLContext> create_gl_context() override;
 
 private:
+    std::shared_ptr<Platform> const platform;
     std::shared_ptr<HostConnection> const connection;
-    std::shared_ptr<input::EventFilter> const event_handler;
+    std::shared_ptr<input::InputDispatcher> const dispatcher;
     std::shared_ptr<DisplayReport> const display_report;
     detail::EGLDisplayHandle egl_display;
 
     std::mutex outputs_mutex;
     std::unordered_map<DisplayConfigurationOutputId, std::shared_ptr<detail::NestedOutput>> outputs;
     DisplayConfigurationChangeHandler my_conf_change_handler;
+    void create_surfaces(mir::graphics::DisplayConfiguration const& configuration);
+    void apply_to_connection(mir::graphics::DisplayConfiguration const& configuration);
     void complete_display_initialization(MirPixelFormat format);
 };
 
