@@ -62,10 +62,11 @@ public:
     DisplayBuffer(SurfacelessEGLContext egl_context,
                   geometry::Rectangle const& area);
 
-    geometry::Rectangle view_area() const;
-    void make_current();
-    void release_current();
-    void post_update();
+    geometry::Rectangle view_area() const override;
+    void make_current() override;
+    void release_current() override;
+    void gl_swap_buffers() override;
+    void flip() override;
 
     MirOrientation orientation() const override;
     bool uses_alpha() const override;
