@@ -41,7 +41,7 @@ class InputRegitrar;
 }
 namespace input
 {
-class InputTargets;
+class Scene;
 namespace android
 {
 class WindowHandleRepository;
@@ -49,15 +49,15 @@ class WindowHandleRepository;
 class InputTargetEnumerator : public droidinput::InputEnumerator
 {
 public:
-    explicit InputTargetEnumerator(std::shared_ptr<input::InputTargets> const& targets,
+    explicit InputTargetEnumerator(std::shared_ptr<input::Scene> const& scene,
                                    std::shared_ptr<WindowHandleRepository> const& repository);
     virtual ~InputTargetEnumerator() noexcept(true);
 
     void for_each(std::function<void(droidinput::sp<droidinput::InputWindowHandle> const&)> const& callback);
 
 private:
-    std::weak_ptr<input::InputTargets> const targets;
-    std::weak_ptr<input::android::WindowHandleRepository> const repository;
+    std::shared_ptr<input::Scene> const scene;
+    std::shared_ptr<input::android::WindowHandleRepository> const repository;
 };
 
 }
