@@ -29,8 +29,8 @@ namespace input
 class NullInputDispatcher : public mir::input::InputDispatcher
 {
 public:
-    void configuration_changed(nsecs_t when) override;
-    void device_reset(int32_t device_id, nsecs_t when) override;
+    void configuration_changed(std::chrono::nanoseconds when) override;
+    void device_reset(int32_t device_id, std::chrono::nanoseconds when) override;
     void dispatch(MirEvent const& event) override;
     void start() override;
     void stop() override;
