@@ -28,7 +28,11 @@ void mrn::CompositorReport::began_frame(SubCompositorId)
 {
 }
 
-void mrn::CompositorReport::finished_frame(bool, SubCompositorId)
+void mrn::CompositorReport::rendered_frame(SubCompositorId)
+{
+}
+
+void mrn::CompositorReport::finished_frame(SubCompositorId)
 {
 }
 
