@@ -44,7 +44,6 @@ public:
     SurfaceAllocator(std::shared_ptr<BufferStreamFactory> const& bb_factory,
                      std::shared_ptr<input::InputChannelFactory> const& input_factory,
                      std::shared_ptr<input::InputSender> const& input_sender,
-                     std::shared_ptr<SurfaceConfigurator> const& configurator,
                      std::shared_ptr<graphics::CursorImage> const& default_cursor_image,
                      std::shared_ptr<SceneReport> const& report);
 
@@ -54,7 +53,6 @@ private:
     std::shared_ptr<BufferStreamFactory> const buffer_stream_factory;
     std::shared_ptr<input::InputChannelFactory> const input_factory;
     std::shared_ptr<input::InputSender> const input_sender;
-    std::shared_ptr<SurfaceConfigurator> const configurator;
     std::shared_ptr<graphics::CursorImage> const default_cursor_image;
     std::shared_ptr<SceneReport> const report;
 };
