@@ -1,5 +1,5 @@
 /*
- * Copyright © 2013-2014 Canonical Ltd.
+ * Copyright © 2013-2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3,
@@ -20,28 +20,23 @@
 #include "surface_stack_model.h"
 #include "mir/scene/surface_factory.h"
 #include "mir/scene/surface.h"
-#include "mir/scene/placement_strategy.h"
 
 namespace ms = mir::scene;
 
 ms::SurfaceController::SurfaceController(
     std::shared_ptr<SurfaceFactory> const& surface_factory,
-    std::shared_ptr<ms::PlacementStrategy> const& placement_strategy,
     std::shared_ptr<SurfaceStackModel> const& surface_stack) :
     surface_factory(surface_factory),
-    placement_strategy(placement_strategy),
     surface_stack(surface_stack)
 {
 }
 
 std::shared_ptr<ms::Surface> ms::SurfaceController::add_surface(
     SurfaceCreationParameters const& params,
-    Session* session)
+    Session* /*session*/)
 {
-    auto placed_params = placement_strategy->place(*session, params);
-
-    auto const surface = surface_factory->create_surface(placed_params);
-    surface_stack->add_surface(surface, placed_params.depth, placed_params.input_mode);
+    auto const surface = surface_factory->create_surface(params);
+    surface_stack->add_surface(surface, params.depth, params.input_mode);
     return surface;
 }
 
