@@ -47,9 +47,11 @@ class SurfaceSceneElement : public mc::SceneElement
 public:
     SurfaceSceneElement(
         std::shared_ptr<mg::Renderable> renderable,
-        std::shared_ptr<ms::RenderingTracker> const& tracker)
+        std::shared_ptr<ms::RenderingTracker> const& tracker,
+        mc::CompositorID id)
         : renderable_{renderable},
-          tracker{tracker}
+          tracker{tracker},
+          cid{id}
     {
     }
 
@@ -58,12 +60,12 @@ public:
         return renderable_;
     }
 
-    void rendered_in(mc::CompositorID cid) override
+    void rendered() override
     {
         tracker->rendered_in(cid);
     }
 
-    void occluded_in(mc::CompositorID cid) override
+    void occluded() override
     {
         tracker->occluded_in(cid);
     }
@@ -76,8 +78,10 @@ public:
 private:
     std::shared_ptr<mg::Renderable> const renderable_;
     std::shared_ptr<ms::RenderingTracker> const tracker;
+    mc::CompositorID cid;
 };
 
+//note: something different than a 2D/HWC overlay
 class OverlaySceneElement : public mc::SceneElement
 {
 public:
@@ -92,11 +96,11 @@ public:
         return renderable_;
     }
 
-    void rendered_in(mc::CompositorID /* cid */) override
+    void rendered() override
     {
     }
 
-    void occluded_in(mc::CompositorID /* cid */) override
+    void occluded() override
     {
     }
     
@@ -120,15 +124,21 @@ ms::SurfaceStack::SurfaceStack(
 mc::SceneElementSequence ms::SurfaceStack::scene_elements_for(mc::CompositorID id)
 {
     std::lock_guard<decltype(guard)> lg(guard);
+
+    scene_changed = false;
     mc::SceneElementSequence elements;
     for (auto const& layer : layers_by_depth)
     {
         for (auto const& surface : layer.second) 
         {
-            auto element = std::make_shared<SurfaceSceneElement>(
-                surface->compositor_snapshot(id),
-                rendering_trackers[surface.get()]);
-            elements.emplace_back(element);
+            if (surface->visible())
+            {
+                auto element = std::make_shared<SurfaceSceneElement>(
+                    surface->compositor_snapshot(id),
+                    rendering_trackers[surface.get()],
+                    id);
+                elements.emplace_back(element);
+            }
         }
     }
     for (auto const& renderable : overlays)
@@ -138,6 +148,29 @@ mc::SceneElementSequence ms::SurfaceStack::scene_elements_for(mc::CompositorID i
     return elements;
 }
 
+int ms::SurfaceStack::frames_pending(mc::CompositorID id) const
+{
+    std::lock_guard<decltype(guard)> lg(guard);
+
+    int result = scene_changed ? 1 : 0;
+    for (auto const& layer : layers_by_depth)
+    {
+        for (auto const& surface : layer.second) 
+        {
+            if (surface->visible())
+            {
+                // Note that we ask the surface and not a Renderable.
+                // This is because we don't want to waste time and resources
+                // on a snapshot till we're sure we need it...
+                int ready = surface->buffers_ready_for_compositor(id);
+                if (ready > result)
+                    result = ready;
+            }
+        }
+    }
+    return result;
+}
+
 void ms::SurfaceStack::register_compositor(mc::CompositorID cid)
 {
     std::lock_guard<decltype(guard)> lg(guard);
@@ -163,7 +196,7 @@ void ms::SurfaceStack::add_input_visualization(
         std::lock_guard<decltype(guard)> lg(guard);
         overlays.push_back(overlay);
     }
-    observers.scene_changed();
+    emit_scene_changed();
 }
 
 void ms::SurfaceStack::remove_input_visualization(
@@ -180,11 +213,15 @@ void ms::SurfaceStack::remove_input_visualization(
         overlays.erase(p);
     }
     
-    observers.scene_changed();
+    emit_scene_changed();
 }
 
 void ms::SurfaceStack::emit_scene_changed()
 {
+    {
+        std::lock_guard<decltype(guard)> lg(guard);
+        scene_changed = true;
+    }
     observers.scene_changed();
 }
 
