@@ -85,15 +85,13 @@ int main(int argc, char *argv[])
     mir_connection_get_available_surface_formats(conn, formats, 1,
                                                  &valid_formats);
 
-    MirSurfaceParameters parm;
-    parm.buffer_usage = mir_buffer_usage_software;
-    parm.output_id = mir_display_output_id_invalid;
-    parm.pixel_format = formats[0];
-    parm.name = "ping";
-    parm.width = 1;
-    parm.height = 1;
+    MirSurfaceSpec *spec = mir_connection_create_spec_for_normal_surface(conn, 1, 1, formats[0]);
+    mir_surface_spec_set_buffer_usage(spec, mir_buffer_usage_software);
+    mir_surface_spec_set_name(spec, "ping");
+
+    MirSurface *surf = mir_surface_create_sync(spec);
+    mir_surface_spec_release(spec);
 
-    MirSurface *surf = mir_connection_create_surface_sync(conn, &parm);
     if (surf == NULL || !mir_surface_is_valid(surf))
     {
         fprintf(stderr, "Could not create a surface.\n");
@@ -105,10 +103,10 @@ int main(int argc, char *argv[])
     signal(SIGTERM, shutdown);
     signal(SIGHUP, shutdown);
 
-    static const MirSurfaceType types[2] =
+    static const MirOrientationMode types[2] =
     {
-        mir_surface_type_normal,
-        mir_surface_type_overlay
+        mir_orientation_mode_portrait_any,
+        mir_orientation_mode_landscape_any
     };
     int t = 1;
 
@@ -118,7 +116,7 @@ int main(int argc, char *argv[])
         long long start, duration;
 
         start = now();
-        mir_wait_for(mir_surface_set_type(surf, types[t]));
+        mir_wait_for(mir_surface_set_preferred_orientation(surf, types[t]));
         duration = now() - start;
         t ^= 1;
 
