@@ -16,14 +16,16 @@
  * Authored by: Thomas Guest <thomas.guest@canonical.com>
  */
 
+#define MIR_INCLUDE_DEPRECATED_EVENT_HEADER
+
 #include "mir_toolkit/mir_client_library.h"
-#include "mir_toolkit/mir_client_library_debug.h"
 
-#include "mir_test_framework/stubbed_server_configuration.h"
-#include "mir_test_framework/in_process_server.h"
+#include "mir_test_framework/headless_in_process_server.h"
 #include "mir_test_framework/using_stub_client_platform.h"
+#include "mir_test_framework/stub_platform_helpers.h"
+#include "mir_test/validity_matchers.h"
 
-#include "src/client/client_buffer.h"
+#include "src/include/client/mir/client_buffer.h"
 
 #include "mir_protobuf.pb.h"
 
@@ -51,15 +53,10 @@ namespace mf = mir::frontend;
 namespace mc = mir::compositor;
 namespace mcl = mir::client;
 namespace mtf = mir_test_framework;
-
 namespace
 {
-struct ClientLibrary : mir_test_framework::InProcessServer
+struct ClientLibrary : mtf::HeadlessInProcessServer
 {
-    mtf::StubbedServerConfiguration server_configuration;
-    mir::DefaultServerConfiguration& server_config() override { return server_configuration; }
-    mtf::UsingStubClientPlatform using_stub_client_platform;
-
     std::set<MirSurface*> surfaces;
     MirConnection* connection = nullptr;
     MirSurface* surface  = nullptr;
@@ -138,16 +135,10 @@ struct ClientLibrary : mir_test_framework::InProcessServer
         {
             mir_wait_for_one(mir_surface_set_state(surf,
                                             mir_surface_state_maximized));
-            mir_wait_for_one(mir_surface_set_type(surf,
-                                            mir_surface_type_normal));
             mir_wait_for_one(mir_surface_set_state(surf,
                                             mir_surface_state_restored));
-            mir_wait_for_one(mir_surface_set_type(surf,
-                                            mir_surface_type_utility));
             mir_wait_for_one(mir_surface_set_state(surf,
                                             mir_surface_state_fullscreen));
-            mir_wait_for_one(mir_surface_set_type(surf,
-                                            mir_surface_type_dialog));
             mir_wait_for_one(mir_surface_set_state(surf,
                                             mir_surface_state_minimized));
         }
@@ -185,16 +176,15 @@ TEST_F(ClientLibrary, creates_surface)
 {
     mir_wait_for(mir_connect(new_connection().c_str(), __PRETTY_FUNCTION__, connection_callback, this));
 
-    MirSurfaceParameters const request_params =
-    {
-        __PRETTY_FUNCTION__,
-        640, 480,
-        mir_pixel_format_abgr_8888,
-        mir_buffer_usage_hardware,
-        mir_display_output_id_invalid
-    };
+    int request_width = 640, request_height = 480;
+    MirPixelFormat request_format = mir_pixel_format_abgr_8888;
+    MirBufferUsage request_buffer_usage = mir_buffer_usage_hardware;
 
-    mir_wait_for(mir_connection_create_surface(connection, &request_params, create_surface_callback, this));
+    auto spec = mir_connection_create_spec_for_normal_surface(connection, request_width,
+                                                              request_height, request_format);
+    mir_surface_spec_set_buffer_usage(spec, request_buffer_usage);
+    surface = mir_surface_create_sync(spec);
+    mir_surface_spec_release(spec);
 
     ASSERT_THAT(surface, NotNull());
     EXPECT_TRUE(mir_surface_is_valid(surface));
@@ -202,31 +192,18 @@ TEST_F(ClientLibrary, creates_surface)
 
     MirSurfaceParameters response_params;
     mir_surface_get_parameters(surface, &response_params);
-    EXPECT_EQ(request_params.width, response_params.width);
-    EXPECT_EQ(request_params.height, response_params.height);
-    EXPECT_EQ(request_params.pixel_format, response_params.pixel_format);
-    EXPECT_EQ(request_params.buffer_usage, response_params.buffer_usage);
+    EXPECT_EQ(request_width, response_params.width);
+    EXPECT_EQ(request_height, response_params.height);
+    EXPECT_EQ(request_format, response_params.pixel_format);
+    EXPECT_EQ(request_buffer_usage, response_params.buffer_usage);
 
     mir_wait_for(mir_surface_release( surface, release_surface_callback, this));
-
-    ASSERT_THAT(surface, IsNull());
-
-    surface = mir_connection_create_surface_sync(connection, &request_params);
-
-    ASSERT_THAT(surface, NotNull());
-    EXPECT_TRUE(mir_surface_is_valid(surface));
-    EXPECT_THAT(mir_surface_get_error_message(surface), StrEq(""));
-
-    mir_surface_get_parameters(surface, &response_params);
-    EXPECT_THAT(response_params.width, Eq(request_params.width));
-    EXPECT_THAT(response_params.height, Eq(request_params.height));
-    EXPECT_THAT(response_params.pixel_format, Eq(request_params.pixel_format));
-    EXPECT_THAT(response_params.buffer_usage, Eq(request_params.buffer_usage));
-
-    mir_surface_release_sync(surface);
     mir_connection_release(connection);
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+
 TEST_F(ClientLibrary, can_set_surface_types)
 {
     mir_wait_for(mir_connect(new_connection().c_str(), __PRETTY_FUNCTION__, connection_callback, this));
@@ -262,15 +239,16 @@ TEST_F(ClientLibrary, can_set_surface_types)
         mir_surface_set_type(surface, mir_surface_type_normal);
         mir_surface_set_type(surface, mir_surface_type_utility);
         mir_surface_set_type(surface, mir_surface_type_dialog);
-        mir_surface_set_type(surface, mir_surface_type_overlay);
+        mir_surface_set_type(surface, mir_surface_type_gloss);
         mir_surface_set_type(surface, mir_surface_type_freestyle);
-        mir_wait_for(mir_surface_set_type(surface, mir_surface_type_popover));
-        ASSERT_THAT(mir_surface_get_type(surface), Eq(mir_surface_type_popover));
+        mir_wait_for(mir_surface_set_type(surface, mir_surface_type_menu));
+        ASSERT_THAT(mir_surface_get_type(surface), Eq(mir_surface_type_menu));
     }
 
     mir_wait_for(mir_surface_release(surface, release_surface_callback, this));
     mir_connection_release(connection);
 }
+#pragma GCC diagnostic pop
 
 TEST_F(ClientLibrary, can_set_surface_state)
 {
@@ -557,8 +535,7 @@ TEST_F(ClientLibrary, highly_threaded_client)
     b.join();
     c.join();
 
-    EXPECT_THAT(mir_surface_get_type(surface), Eq(mir_surface_type_dialog));
-    EXPECT_THAT(mir_surface_get_type(surface), Eq(mir_surface_state_minimized));
+    EXPECT_THAT(mir_surface_get_state(surface), Eq(mir_surface_state_minimized));
 
     mir_surface_release_sync(surface);
 
@@ -569,15 +546,14 @@ TEST_F(ClientLibrary, highly_threaded_client)
 
 TEST_F(ClientLibrary, accesses_platform_package)
 {
+    using namespace testing;
     mir_wait_for(mir_connect(new_connection().c_str(), __PRETTY_FUNCTION__, connection_callback, this));
 
     MirPlatformPackage platform_package;
-    platform_package.data_items = -1;
-    platform_package.fd_items = -1;
+    ::memset(&platform_package, -1, sizeof(platform_package));
 
     mir_connection_get_platform(connection, &platform_package);
-    EXPECT_GE(0, platform_package.data_items);
-    EXPECT_GE(0, platform_package.fd_items);
+    EXPECT_THAT(platform_package, mtf::IsStubPlatformPackage());
 
     mir_connection_release(connection);
 }
@@ -602,42 +578,6 @@ TEST_F(ClientLibrary, accesses_display_info)
     mir_connection_release(connection);
 }
 
-TEST_F(ClientLibrary, connect_errors_handled)
-{
-    mir_wait_for(mir_connect("garbage", __PRETTY_FUNCTION__, connection_callback, this));
-    ASSERT_THAT(connection, NotNull());
-
-    char const* error = mir_connection_get_error_message(connection);
-
-    if (std::strcmp("connect: No such file or directory", error) &&
-        std::strcmp("Can't find MIR server", error) &&
-        !std::strstr(error, "Failed to connect to server socket"))
-    {
-        FAIL() << error;
-    }
-}
-
-TEST_F(ClientLibrary, connect_errors_dont_blow_up)
-{
-    mir_wait_for(mir_connect("garbage", __PRETTY_FUNCTION__, connection_callback, this));
-
-    MirSurfaceParameters const request_params =
-    {
-        __PRETTY_FUNCTION__,
-        640, 480,
-        mir_pixel_format_abgr_8888,
-        mir_buffer_usage_hardware,
-        mir_display_output_id_invalid
-    };
-
-    mir_wait_for(mir_connection_create_surface(connection, &request_params, create_surface_callback, this));
-// TODO surface_create needs to fail safe too. After that is done we should add the following:
-// TODO    mir_wait_for(mir_surface_swap_buffers(surface, next_buffer_callback, this));
-// TODO    mir_wait_for(mir_surface_release( surface, release_surface_callback, this));
-
-    mir_connection_release(connection);
-}
-
 TEST_F(ClientLibrary, MultiSurfaceClientTracksBufferFdsCorrectly)
 {
     mir_wait_for(mir_connect(new_connection().c_str(), __PRETTY_FUNCTION__, connection_callback, this));
@@ -684,3 +624,217 @@ TEST_F(ClientLibrary, MultiSurfaceClientTracksBufferFdsCorrectly)
 
     mir_connection_release(connection);
 }
+
+/* TODO: Our stub platform support is a bit terrible.
+ *
+ * These acceptance tests accidentally work on mesa because the mesa client
+ * platform doesn't validate any of its input and we don't touch anything that requires
+ * syscalls.
+ *
+ * The Android client platform *does* care about its input, and so the fact that it's
+ * trying to marshall stub buffers causes crashes.
+ */
+
+#ifndef ANDROID
+TEST_F(ClientLibrary, create_simple_normal_surface_from_spec)
+#else
+TEST_F(ClientLibrary, DISABLED_create_simple_normal_surface_from_spec)
+#endif
+{
+    auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
+
+    int const width{800}, height{600};
+    MirPixelFormat const format{mir_pixel_format_bgr_888};
+    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection,
+                                                                      width, height,
+                                                                      format);
+
+    auto surface = mir_surface_create_sync(surface_spec);
+    mir_surface_spec_release(surface_spec);
+
+    EXPECT_THAT(surface, IsValid());
+
+    MirNativeBuffer* native_buffer;
+    mir_surface_get_current_buffer(surface, &native_buffer);
+
+    EXPECT_THAT(native_buffer->width, Eq(width));
+    EXPECT_THAT(native_buffer->height, Eq(height));
+    EXPECT_THAT(mir_surface_get_type(surface), Eq(mir_surface_type_normal));
+
+    mir_surface_release_sync(surface);
+    mir_connection_release(connection);
+}
+
+#ifndef ANDROID
+TEST_F(ClientLibrary, create_simple_normal_surface_from_spec_async)
+#else
+TEST_F(ClientLibrary, DISABLED_create_simple_normal_surface_from_spec_async)
+#endif
+{
+    auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
+
+    int const width{800}, height{600};
+    MirPixelFormat const format{mir_pixel_format_xbgr_8888};
+    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection,
+                                                                      width, height,
+                                                                      format);
+
+    mir_wait_for(mir_surface_create(surface_spec, create_surface_callback, this));
+    mir_surface_spec_release(surface_spec);
+
+    EXPECT_THAT(surface, IsValid());
+
+    MirNativeBuffer* native_buffer;
+    mir_surface_get_current_buffer(surface, &native_buffer);
+
+    EXPECT_THAT(native_buffer->width, Eq(width));
+    EXPECT_THAT(native_buffer->height, Eq(height));
+    EXPECT_THAT(mir_surface_get_type(surface), Eq(mir_surface_type_normal));
+
+    mir_surface_release_sync(surface);
+    mir_connection_release(connection);
+}
+
+#ifndef ANDROID
+TEST_F(ClientLibrary, can_specify_all_normal_surface_parameters_from_spec)
+#else
+TEST_F(ClientLibrary, DISABLED_can_specify_all_normal_surface_parameters_from_spec)
+#endif
+{
+    using namespace testing;
+
+    auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
+
+    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection,
+                                                                      800, 600,
+                                                                      mir_pixel_format_bgr_888);
+
+    char const* name = "The magnificent Dandy Warhols";
+    EXPECT_TRUE(mir_surface_spec_set_name(surface_spec, name));
+
+    int const width{999}, height{555};
+    EXPECT_TRUE(mir_surface_spec_set_width(surface_spec, width));
+    EXPECT_TRUE(mir_surface_spec_set_height(surface_spec, height));
+
+    MirPixelFormat const pixel_format{mir_pixel_format_argb_8888};
+    EXPECT_TRUE(mir_surface_spec_set_pixel_format(surface_spec, pixel_format));
+
+    MirBufferUsage const buffer_usage{mir_buffer_usage_hardware};
+    EXPECT_TRUE(mir_surface_spec_set_buffer_usage(surface_spec, buffer_usage));
+
+    auto surface = mir_surface_create_sync(surface_spec);
+    mir_surface_spec_release(surface_spec);
+
+    EXPECT_THAT(surface, IsValid());
+
+    mir_surface_release_sync(surface);
+    mir_connection_release(connection);
+}
+
+#ifndef ANDROID
+TEST_F(ClientLibrary, set_fullscreen_on_output_makes_fullscreen_surface)
+#else
+TEST_F(ClientLibrary, DISABLED_set_fullscreen_on_output_makes_fullscreen_surface)
+#endif
+{
+    using namespace testing;
+
+    auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
+
+    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection,
+                                                                      780, 555,
+                                                                      mir_pixel_format_xbgr_8888);
+
+    // We need to specify a valid output id, so we need to find which ones are valid...
+    auto configuration = mir_connection_create_display_config(connection);
+    ASSERT_THAT(configuration->num_outputs, Ge(1));
+
+    auto const requested_output = configuration->outputs[0];
+
+    mir_surface_spec_set_fullscreen_on_output(surface_spec, requested_output.output_id);
+
+    auto surface = mir_surface_create_sync(surface_spec);
+    mir_surface_spec_release(surface_spec);
+
+    EXPECT_THAT(surface, IsValid());
+
+    MirNativeBuffer* native_buffer;
+    mir_surface_get_current_buffer(surface, &native_buffer);
+
+    EXPECT_THAT(native_buffer->width,
+                Eq(requested_output.modes[requested_output.current_mode].horizontal_resolution));
+    EXPECT_THAT(native_buffer->height,
+                Eq(requested_output.modes[requested_output.current_mode].vertical_resolution));
+
+// TODO: This is racy. Fix in subsequent "send all the things on construction" branch
+//    EXPECT_THAT(mir_surface_get_state(surface), Eq(mir_surface_state_fullscreen));
+
+    mir_surface_release_sync(surface);
+    mir_display_config_destroy(configuration);
+    mir_connection_release(connection);
+}
+
+/*
+ * We don't (yet) use a stub client platform, so can't rely on its behaviour
+ * in these tests.
+ *
+ * At the moment, enabling them will either spuriously pass (hardware buffer, mesa)
+ * or crash (everything else).
+ */
+TEST_F(ClientLibrary, DISABLED_can_create_buffer_usage_hardware_surface)
+{
+    using namespace testing;
+
+    auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
+
+    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection,
+                                                                      800, 600,
+                                                                      mir_pixel_format_bgr_888);
+
+    MirBufferUsage const buffer_usage{mir_buffer_usage_hardware};
+    EXPECT_TRUE(mir_surface_spec_set_buffer_usage(surface_spec, buffer_usage));
+
+    auto surface = mir_surface_create_sync(surface_spec);
+    mir_surface_spec_release(surface_spec);
+
+    EXPECT_THAT(surface, IsValid());
+
+    MirNativeBuffer* native_buffer;
+    // We use the fact that our stub client platform returns NULL if asked for a native
+    // buffer on a surface with mir_buffer_usage_software set.
+    mir_surface_get_current_buffer(surface, &native_buffer);
+
+    EXPECT_THAT(native_buffer, Not(Eq(nullptr)));
+
+    mir_surface_release_sync(surface);
+    mir_connection_release(connection);
+}
+
+TEST_F(ClientLibrary, DISABLED_can_create_buffer_usage_software_surface)
+{
+    using namespace testing;
+
+    auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
+
+    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection,
+                                                                      800, 600,
+                                                                      mir_pixel_format_bgr_888);
+
+    MirBufferUsage const buffer_usage{mir_buffer_usage_software};
+    EXPECT_TRUE(mir_surface_spec_set_buffer_usage(surface_spec, buffer_usage));
+
+    auto surface = mir_surface_create_sync(surface_spec);
+    mir_surface_spec_release(surface_spec);
+
+    EXPECT_THAT(surface, IsValid());
+
+    MirGraphicsRegion graphics_region;
+    // We use the fact that our stub client platform returns a NULL vaddr if
+    // asked to map a hardware buffer.
+    mir_surface_get_graphics_region(surface, &graphics_region);
+
+    EXPECT_THAT(graphics_region.vaddr, Not(Eq(nullptr)));
+
+    mir_surface_release_sync(surface);
+    mir_connection_release(connection);
+}
