@@ -40,20 +40,12 @@ public:
     {
     }
 
-    void begin() const override
-    {
-    }
-
     void render(graphics::RenderableList const& renderables) const override
     {
         for (auto const& r : renderables)
             r->buffer(); // We need to consume a buffer to unblock client tests
     }
 
-    void end() const override
-    {
-    }
-
     void suspend() override
     {
     }
