@@ -17,7 +17,7 @@
  */
 
 #include "mir/default_server_configuration.h"
-#include "src/server/input/null_input_configuration.h"
+#include "src/server/input/null_input_manager.h"
 #include "mir/compositor/compositor.h"
 #include "src/server/scene/application_session.h"
 #include "src/server/scene/pixel_buffer.h"
@@ -37,6 +37,7 @@
 #include "mir_test_doubles/null_pixel_buffer.h"
 
 #include <gtest/gtest.h>
+#include <condition_variable>
 
 namespace mc = mir::compositor;
 namespace mtd = mir::test::doubles;
@@ -54,12 +55,12 @@ struct TestServerConfiguration : public mir::DefaultServerConfiguration
 {
     TestServerConfiguration() : DefaultServerConfiguration(0, nullptr) {}
 
-    std::shared_ptr<mi::InputConfiguration> the_input_configuration() override
+    std::shared_ptr<mi::InputManager> the_input_manager() override
     {
-        if (!input_configuration)
-            input_configuration = std::make_shared<mi::NullInputConfiguration>();
+        if (!input_manager)
+            input_manager = std::make_shared<mi::NullInputManager>();
 
-        return input_configuration;
+        return input_manager;
     }
 
     std::shared_ptr<mf::Connector> the_connector() override
@@ -123,11 +124,31 @@ struct TestServerConfiguration : public mir::DefaultServerConfiguration
             });
     }
 
-    std::shared_ptr<mi::NullInputConfiguration> input_configuration;
+    std::shared_ptr<mi::NullInputManager> input_manager;
 };
 
+void swap_buffers_blocking(mf::Surface& surf, mg::Buffer*& buffer)
+{
+    std::mutex mutex;
+    std::condition_variable cv;
+    bool done = false;
+
+    surf.swap_buffers(buffer,
+        [&](mg::Buffer* new_buffer)
+        {
+            std::unique_lock<decltype(mutex)> lock(mutex);
+            buffer = new_buffer;
+            done = true;
+            cv.notify_one();
+        });
+
+    std::unique_lock<decltype(mutex)> lock(mutex);
+
+    cv.wait(lock, [&]{ return done; });
 }
 
+} // anonymouse namespace
+
 TEST(ApplicationSession, stress_test_take_snapshot)
 {
     TestServerConfiguration conf;
@@ -154,7 +175,7 @@ TEST(ApplicationSession, stress_test_take_snapshot)
             for (int i = 0; i < 500; ++i)
             {
                 auto surface = session.default_surface();
-                surface->swap_buffers_blocking(buffer);
+                swap_buffers_blocking(*surface, buffer);
                 std::this_thread::sleep_for(std::chrono::microseconds{50});
             }
         }};
