@@ -21,34 +21,38 @@
 #include <boost/throw_exception.hpp>
 #include <boost/exception/errinfo_errno.hpp>
 
-#include <stdexcept>
+#include <system_error>
 
 #include <unistd.h>
+#include <fcntl.h>
 
 namespace mt = mir::test;
 
 mt::Pipe::Pipe()
+    : Pipe(0)
 {
-    if (pipe(pipefd))
+}
+
+mt::Pipe::Pipe(int flags)
+{
+    int pipefd[2];
+    if (pipe2(pipefd, flags))
     {
         BOOST_THROW_EXCEPTION(
-            boost::enable_error_info(std::runtime_error("Failed to create pipe"))
-                << boost::errinfo_errno(errno));
+            boost::enable_error_info(std::system_error(errno,
+                                                       std::system_category(),
+                                                       "Failed to create pipe")));
     }
+    reader = mir::Fd{pipefd[0]};
+    writer = mir::Fd{pipefd[1]};
 }
 
-mt::Pipe::~Pipe()
+mir::Fd mt::Pipe::read_fd() const
 {
-    close(pipefd[0]);
-    close(pipefd[1]);
+    return reader;
 }
 
-int mt::Pipe::read_fd() const
+mir::Fd mt::Pipe::write_fd() const
 {
-    return pipefd[0];
-}
-
-int mt::Pipe::write_fd() const
-{
-    return pipefd[1];
+    return writer;
 }
