@@ -20,6 +20,8 @@
 #include "mir_test_doubles/mock_gl.h"
 #include <gtest/gtest.h>
 
+#include <cstring>
+
 namespace mtd = mir::test::doubles;
 
 namespace
@@ -36,6 +38,25 @@ mtd::MockGL::MockGL()
 
     ON_CALL(*this, glCheckFramebufferStatus(_))
         .WillByDefault(Return(GL_FRAMEBUFFER_COMPLETE));
+    ON_CALL(*this, glGetShaderiv(_,_,_))
+        .WillByDefault(SetArgPointee<2>(GL_TRUE));
+    ON_CALL(*this, glGetProgramiv(_,_,_))
+        .WillByDefault(SetArgPointee<2>(GL_TRUE));
+    ON_CALL(*this, glGenTextures(_, _))
+        .WillByDefault(Invoke(
+            [] (GLsizei n, GLuint *textures)
+            {
+                std::memset(textures, 0, n * sizeof(*textures));
+            }));
+}
+
+void mtd::MockGL::provide_gles_extensions()
+{
+    using namespace testing;
+    const char* gl_exts = "GL_OES_EGL_image";
+
+    ON_CALL(*this, glGetString(GL_EXTENSIONS))
+        .WillByDefault(Return(reinterpret_cast<const GLubyte*>(gl_exts)));
 }
 
 mtd::MockGL::~MockGL()
@@ -84,6 +105,18 @@ void glClear (GLbitfield mask)
     global_mock_gl->glClear(mask);
 }
 
+void glClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
+{
+    CHECK_GLOBAL_VOID_MOCK();
+    global_mock_gl->glClearColor(red, green, blue, alpha);
+}
+
+void glColorMask(GLboolean r, GLboolean g, GLboolean b, GLboolean a)
+{
+    CHECK_GLOBAL_VOID_MOCK();
+    global_mock_gl->glColorMask(r, g, b, a);
+}
+
 void glEnable(GLenum func)
 {
     CHECK_GLOBAL_VOID_MOCK();
@@ -403,3 +436,9 @@ void glFinish()
     CHECK_GLOBAL_VOID_MOCK();
     global_mock_gl->glFinish();
 }
+
+void glGenerateMipmap(GLenum target)
+{
+    CHECK_GLOBAL_VOID_MOCK();
+    global_mock_gl->glGenerateMipmap(target);
+}
