@@ -21,12 +21,10 @@
 #include "mir/input/input_channel.h"
 #include "mir/input/surface.h"
 #include "mir/scene/surface_creation_parameters.h"
-#include "mir/frontend/shell.h"
 #include "mir/frontend/session.h"
 #include "mir/input/composite_event_filter.h"
 
 #include "mir_test/fake_event_hub.h"
-#include "mir_test/fake_event_hub_input_configuration.h"
 #include "mir_test/wait_condition.h"
 
 #include <boost/throw_exception.hpp>
@@ -84,6 +82,11 @@ std::shared_ptr<ms::InputTargeter> mtf::InputTestingServerConfiguration::the_inp
     return DefaultServerConfiguration::the_input_targeter();
 }
 
+std::shared_ptr<mi::InputManager> mtf::InputTestingServerConfiguration::the_input_manager()
+{
+    return DefaultServerConfiguration::the_input_manager();
+}
+
 std::shared_ptr<mi::InputDispatcher> mtf::InputTestingServerConfiguration::the_input_dispatcher()
 {
     return DefaultServerConfiguration::the_input_dispatcher();
@@ -94,17 +97,11 @@ std::shared_ptr<mi::InputSender> mtf::InputTestingServerConfiguration::the_input
     return DefaultServerConfiguration::the_input_sender();
 }
 
-std::shared_ptr<mi::InputConfiguration> mtf::InputTestingServerConfiguration::the_input_configuration()
+std::shared_ptr<droidinput::EventHubInterface> mtf::InputTestingServerConfiguration::the_event_hub()
 {
-    if (!input_configuration)
+    if (!fake_event_hub)
     {
-        input_configuration = std::make_shared<mtd::FakeEventHubInputConfiguration>(
-            the_input_dispatcher(),
-            the_input_region(),
-            the_cursor_listener(),
-            the_touch_visualizer(),
-            the_input_report());
-        fake_event_hub = input_configuration->the_fake_event_hub();
+        fake_event_hub = std::make_shared<mia::FakeEventHub>();
 
         fake_event_hub->synthesize_builtin_keyboard_added();
         fake_event_hub->synthesize_builtin_cursor_added();
@@ -112,5 +109,5 @@ std::shared_ptr<mi::InputConfiguration> mtf::InputTestingServerConfiguration::th
         fake_event_hub->synthesize_device_scan_complete();
     }
 
-    return input_configuration;
+    return fake_event_hub;
 }
