@@ -16,8 +16,10 @@
  * Authored by: Robert Carr <robert.carr@canonical.com>
  */
 
-#include "src/shared/input/android/android_input_receiver_thread.h"
-#include "src/shared/input/android/android_input_receiver.h"
+#define MIR_INCLUDE_DEPRECATED_EVENT_HEADER
+
+#include "src/common/input/android/android_input_receiver_thread.h"
+#include "src/common/input/android/android_input_receiver.h"
 
 #include "mir/input/null_input_receiver_report.h"
 
