@@ -16,9 +16,9 @@
  * Authored by: Kevin DuBois <kevin.dubois@canonical.com>
  */
 
-#include "src/client/mesa/native_surface.h"
-#include "src/client/client_buffer.h"
-#include "mir_test_doubles/mock_client_surface.h"
+#include "src/platforms/mesa/client/native_surface.h"
+#include "mir/client_buffer.h"
+#include "mir_test_doubles/mock_egl_native_surface.h"
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -50,6 +50,7 @@ struct MockClientBuffer : public mcl::ClientBuffer
     MOCK_METHOD0(mark_as_submitted, void());
     MOCK_METHOD0(increment_age, void());
     MOCK_METHOD1(update_from, void(MirBufferPackage const&));
+    MOCK_METHOD1(fill_update_msg, void(MirBufferPackage&));
     MOCK_CONST_METHOD0(native_buffer_handle, std::shared_ptr<MirNativeBuffer>());
 };
 
@@ -73,7 +74,7 @@ public:
     }
 
     MirSurfaceParameters surf_params;
-    testing::NiceMock<mtd::MockClientSurface> mock_surface;
+    testing::NiceMock<mtd::MockEGLNativeSurface> mock_surface;
     mclg::NativeSurface native_surface{mock_surface};
 };
 
