@@ -128,7 +128,19 @@ public:
         throw_on_add_observer_ = flag;
     }
 
+    int frames_pending(mc::CompositorID) const override
+    {
+        return pending;
+    }
+
+    void set_pending(int n)
+    {
+        pending = n;
+        observer->scene_changed();
+    }
+
 private:
+    int pending = 0;
     std::mutex observer_mutex;
     std::shared_ptr<ms::Observer> observer;
     bool throw_on_add_observer_;
@@ -142,7 +154,7 @@ public:
     {
     }
 
-    void composite()
+    void composite(mc::SceneElementSequence&&)
     {
         mark_render_buffer();
         /* Reduce run-time under valgrind */
@@ -258,7 +270,7 @@ public:
     {
     }
 
-    void composite()
+    void composite(mc::SceneElementSequence&&) override
     {
         fake_surface_update();
         /* Reduce run-time under valgrind */
@@ -307,7 +319,9 @@ public:
     {
         struct NullDisplayBufferCompositor : mc::DisplayBufferCompositor
         {
-            void composite() {}
+            void composite(mc::SceneElementSequence&&) override
+            {
+            }
         };
 
         auto raw = new NullDisplayBufferCompositor{};
@@ -478,6 +492,79 @@ TEST(MultiThreadedCompositor, composites_only_on_demand)
     compositor.stop();
 }
 
+TEST(MultiThreadedCompositor, schedules_enough_frames)
+{
+    using namespace testing;
+
+    unsigned int const nbuffers = 3;
+
+    auto display = std::make_shared<StubDisplay>(nbuffers);
+    auto scene = std::make_shared<StubScene>();
+    auto factory = std::make_shared<RecordingDisplayBufferCompositorFactory>();
+    mc::MultiThreadedCompositor compositor{display, scene, factory,
+                                           null_report, true};
+
+    EXPECT_TRUE(factory->check_record_count_for_each_buffer(nbuffers, 0, 0));
+
+    compositor.start();
+
+    int const max_retries = 100;
+    int retry = 0;
+    while (retry < max_retries &&
+           !factory->check_record_count_for_each_buffer(nbuffers, 1))
+    {
+        std::this_thread::sleep_for(std::chrono::milliseconds(10));
+        ++retry;
+    }
+
+    ASSERT_LT(retry, max_retries);
+
+    // Now we have the initial frame wait a bit
+    std::this_thread::sleep_for(std::chrono::milliseconds(100));
+    // ... and make sure the number is still only 1
+    ASSERT_TRUE(factory->check_record_count_for_each_buffer(nbuffers, 1, 1));
+
+    // Trigger scene changes. Only need one frame to cover them all...
+    scene->emit_change_event();
+    scene->emit_change_event();
+    scene->emit_change_event();
+
+    // Display buffers should be forced to render another 1, so that's 2
+    retry = 0;
+    while (retry < max_retries &&
+           !factory->check_record_count_for_each_buffer(nbuffers, 2))
+    {
+        std::this_thread::sleep_for(std::chrono::milliseconds(10));
+        ++retry;
+    }
+    ASSERT_LT(retry, max_retries);
+
+    // Scene unchanged. Expect no new frames:
+    retry = 0;
+    while (retry < max_retries &&
+           !factory->check_record_count_for_each_buffer(nbuffers, 2))
+    {
+        std::this_thread::sleep_for(std::chrono::milliseconds(10));
+        ++retry;
+    }
+    ASSERT_LT(retry, max_retries);
+
+    // Some surface queues up multiple frames
+    scene->set_pending(3);
+
+    // Make sure they all get composited separately (2 + 3 more)
+    retry = 0;
+    while (retry < max_retries &&
+           !factory->check_record_count_for_each_buffer(nbuffers, 5))
+    {
+        std::this_thread::sleep_for(std::chrono::milliseconds(10));
+        ++retry;
+    }
+    ASSERT_LT(retry, max_retries);
+
+    compositor.stop();
+}
+
 TEST(MultiThreadedCompositor, when_no_initial_composite_is_needed_there_is_none)
 {
     using namespace testing;
@@ -681,3 +768,22 @@ TEST(MultiThreadedCompositor, names_compositor_threads)
     for (size_t i = 0; i < thread_names.size(); ++i)
         EXPECT_THAT(thread_names[i], Eq("Mir/Comp")) << "i=" << i;
 }
+
+TEST(MultiThreadedCompositor, registers_and_unregisters_with_scene)
+{
+    using namespace testing;
+    unsigned int const nbuffers{3};
+    auto display = std::make_shared<StubDisplayWithMockBuffers>(nbuffers);
+    auto mock_scene = std::make_shared<NiceMock<mtd::MockScene>>();
+    auto db_compositor_factory = std::make_shared<NullDisplayBufferCompositorFactory>();
+    auto mock_report = std::make_shared<testing::NiceMock<mtd::MockCompositorReport>>();
+
+    EXPECT_CALL(*mock_scene, register_compositor(_))
+        .Times(nbuffers);
+    EXPECT_CALL(*mock_scene, unregister_compositor(_))
+        .Times(nbuffers);
+    mc::MultiThreadedCompositor compositor{display, mock_scene, db_compositor_factory, mock_report, true};
+
+    compositor.start();
+    compositor.stop();
+}
