@@ -27,6 +27,7 @@
 #include "mir/graphics/platform.h"
 #include "mir/graphics/platform_ipc_package.h"
 #include "mir/graphics/buffer_ipc_message.h"
+#include "mir/graphics/platform_operation_message.h"
 #include "mir/input/cursor_images.h"
 #include "mir/graphics/platform_ipc_operations.h"
 #include "src/server/scene/basic_surface.h"
@@ -50,6 +51,8 @@
 #include "mir/frontend/event_sink.h"
 
 #include "gmock_set_arg.h"
+#include <boost/exception/errinfo_errno.hpp>
+#include <boost/throw_exception.hpp>
 #include <gtest/gtest.h>
 #include <gmock/gmock.h>
 
@@ -124,6 +127,7 @@ struct MockBufferPacker : public mg::PlatformIpcOperations
     MOCK_CONST_METHOD2(unpack_buffer,
         void(mg::BufferIpcMessage&, mg::Buffer const&));
     MOCK_METHOD0(connection_ipc_package, std::shared_ptr<mg::PlatformIPCPackage>());
+    MOCK_METHOD2(platform_operation, mg::PlatformOperationMessage(unsigned int const, mg::PlatformOperationMessage const&));
 };
 
 class StubbedSession : public mtd::StubSession
@@ -167,18 +171,18 @@ public:
         return surface;
     }
 
-    mf::SurfaceId create_surface(ms::SurfaceCreationParameters const& /* params */) override
+    mf::SurfaceId create_surface(ms::SurfaceCreationParameters const& /* params */)
     {
         mf::SurfaceId id{last_surface_id};
         if (mock_surfaces.end() == mock_surfaces.find(id))
         {
-            mock_surfaces[id] = create_mock_surface(); 
+            mock_surfaces[id] = create_mock_surface();
         }
         last_surface_id++;
         return id;
     }
 
-    void destroy_surface(mf::SurfaceId surface) override
+    void destroy_surface(mf::SurfaceId surface)
     {
         mock_surfaces.erase(surface);
     }
@@ -202,36 +206,6 @@ public:
     MOCK_METHOD0(supported_pixel_formats, std::vector<MirPixelFormat>());
 };
 
-
-class MockPlatform : public mg::Platform
-{
- public:
-    MockPlatform(std::shared_ptr<mg::PlatformIpcOperations> const& ipc_ops)
-    {
-        using namespace testing;
-        ON_CALL(*this, create_buffer_allocator(_))
-            .WillByDefault(Return(std::shared_ptr<mg::GraphicBufferAllocator>()));
-        ON_CALL(*this, create_display(_,_,_))
-            .WillByDefault(Return(std::make_shared<mtd::NullDisplay>()));
-        ON_CALL(*this, connection_ipc_package())
-            .WillByDefault(Return(std::make_shared<mg::PlatformIPCPackage>()));
-        ON_CALL(*this, make_ipc_operations())
-            .WillByDefault(Return(ipc_ops));
-    }
-
-    MOCK_METHOD1(create_buffer_allocator, std::shared_ptr<mg::GraphicBufferAllocator>(std::shared_ptr<mg::BufferInitializer> const&));
-    MOCK_METHOD3(create_display,
-                 std::shared_ptr<mg::Display>(
-                     std::shared_ptr<mg::DisplayConfigurationPolicy> const&,
-                     std::shared_ptr<mg::GLProgramFactory> const&,
-                     std::shared_ptr<mg::GLConfig> const&));
-    MOCK_METHOD0(connection_ipc_package, std::shared_ptr<mg::PlatformIPCPackage>());
-    MOCK_METHOD0(create_internal_client, std::shared_ptr<mg::InternalClient>());
-    MOCK_CONST_METHOD0(make_ipc_operations, std::shared_ptr<mg::PlatformIpcOperations>());
-    MOCK_CONST_METHOD0(egl_native_display, EGLNativeDisplayType());
-    MOCK_METHOD0(make_buffer_writer, std::shared_ptr<mg::BufferWriter>());
-};
-
 struct StubScreencast : mtd::NullScreencast
 {
     std::shared_ptr<mg::Buffer> capture(mf::ScreencastSessionId)
@@ -246,8 +220,6 @@ struct SessionMediator : public ::testing::Test
 {
     SessionMediator()
         : shell{std::make_shared<testing::NiceMock<mtd::MockShell>>()},
-          graphics_platform{
-              std::make_shared<testing::NiceMock<MockPlatform>>(mt::fake_shared(mock_ipc_operations))},
           graphics_changer{std::make_shared<mtd::NullDisplayChanger>()},
           surface_pixel_formats{mir_pixel_format_argb_8888, mir_pixel_format_xrgb_8888},
           report{mr::null_session_mediator_report()},
@@ -256,20 +228,25 @@ struct SessionMediator : public ::testing::Test
           stubbed_session{std::make_shared<StubbedSession>()},
           null_callback{google::protobuf::NewPermanentCallback(google::protobuf::DoNothing)},
           mediator{
-            shell, graphics_platform, graphics_changer,
+            shell, mt::fake_shared(mock_ipc_operations), graphics_changer,
             surface_pixel_formats, report,
             std::make_shared<mtd::NullEventSink>(),
-            resource_cache, stub_screencast, &connector, nullptr}
+            resource_cache, stub_screencast, &connector, nullptr, nullptr}
     {
         using namespace ::testing;
 
         ON_CALL(*shell, open_session(_, _, _)).WillByDefault(Return(stubbed_session));
+
+        ON_CALL(*shell, create_surface( _, _)).WillByDefault(
+            WithArg<1>(Invoke(stubbed_session.get(), &StubbedSession::create_surface)));
+
+        ON_CALL(*shell, destroy_surface( _, _)).WillByDefault(
+            WithArg<1>(Invoke(stubbed_session.get(), &StubbedSession::destroy_surface)));
     }
 
     MockConnector connector;
     testing::NiceMock<MockBufferPacker> mock_ipc_operations;
     std::shared_ptr<testing::NiceMock<mtd::MockShell>> const shell;
-    std::shared_ptr<MockPlatform> const graphics_platform;
     std::shared_ptr<mf::DisplayChanger> const graphics_changer;
     std::vector<MirPixelFormat> const surface_pixel_formats;
     std::shared_ptr<mf::SessionMediatorReport> const report;
@@ -313,10 +290,10 @@ TEST_F(SessionMediator, connect_calls_connect_handler)
     };
 
     mf::SessionMediator mediator{
-        shell, graphics_platform, graphics_changer,
+        shell, mt::fake_shared(mock_ipc_operations), graphics_changer,
         surface_pixel_formats, report,
         std::make_shared<mtd::NullEventSink>(),
-        resource_cache, stub_screencast, context, nullptr};
+        resource_cache, stub_screencast, context, nullptr, nullptr};
 
     EXPECT_THAT(connects_handled_count, Eq(0));
 
@@ -360,8 +337,10 @@ TEST_F(SessionMediator, calling_methods_after_connect_works)
 
     EXPECT_NO_THROW({
         mediator.create_surface(nullptr, &surface_parameters, &surface_response, null_callback.get());
-        surface_id_request = surface_response.id();
-        mediator.next_buffer(nullptr, &surface_id_request, &buffer_response, null_callback.get());
+        *buffer_request.mutable_buffer() = surface_response.buffer_stream().buffer();
+        *buffer_request.mutable_id() = surface_response.id();
+        mediator.next_buffer(nullptr, buffer_request.mutable_id(), &buffer_response, null_callback.get());
+        mediator.exchange_buffer(nullptr, &buffer_request, &buffer_response, null_callback.get());
         mediator.release_surface(nullptr, &surface_id_request, nullptr, null_callback.get());
     });
 
@@ -415,11 +394,11 @@ TEST_F(SessionMediator, connect_packs_display_configuration)
         .WillByDefault(Return(mt::fake_shared(config)));
 
     mf::SessionMediator mediator(
-        shell, graphics_platform, mock_display,
+        shell, mt::fake_shared(mock_ipc_operations), mock_display,
         surface_pixel_formats, report,
         std::make_shared<mtd::NullEventSink>(),
         resource_cache, std::make_shared<mtd::NullScreencast>(),
-        nullptr, nullptr);
+        nullptr, nullptr, nullptr);
     mediator.connect(nullptr, &connect_parameters, &connection, null_callback.get());
 
     EXPECT_THAT(connection.display_configuration(), mt::DisplayConfigMatches(std::cref(config)));
@@ -476,24 +455,18 @@ TEST_F(SessionMediator, session_only_sends_mininum_information_for_buffers)
     EXPECT_CALL(mock_ipc_operations, pack_buffer(_, Ref(buffer2), mg::BufferIpcMsgType::full_msg))
         .InSequence(seq);
     //swap1
-    EXPECT_CALL(mock_ipc_operations, unpack_buffer(_, Ref(buffer2)))
-        .InSequence(seq);
     EXPECT_CALL(*surface, swap_buffers(&buffer2, _))
         .InSequence(seq)
         .WillOnce(InvokeArgument<1>(&buffer1));
     EXPECT_CALL(mock_ipc_operations, pack_buffer(_, Ref(buffer1), mg::BufferIpcMsgType::full_msg))
         .InSequence(seq);
     //swap2
-    EXPECT_CALL(mock_ipc_operations, unpack_buffer(_, Ref(buffer1)))
-        .InSequence(seq);
     EXPECT_CALL(*surface, swap_buffers(&buffer1, _))
         .InSequence(seq)
         .WillOnce(InvokeArgument<1>(&buffer2));
     EXPECT_CALL(mock_ipc_operations, pack_buffer(_, Ref(buffer2), mg::BufferIpcMsgType::update_msg))
         .InSequence(seq);
     //swap3
-    EXPECT_CALL(mock_ipc_operations, unpack_buffer(_, Ref(buffer2)))
-        .InSequence(seq);
     EXPECT_CALL(*surface, swap_buffers(&buffer2, _))
         .InSequence(seq)
         .WillOnce(InvokeArgument<1>(&buffer1));
@@ -623,11 +596,11 @@ TEST_F(SessionMediator, display_config_request)
         .WillOnce(Return(mt::fake_shared(stub_display_config)));
 
     mf::SessionMediator mediator{
-        shell, graphics_platform, mock_display_selector,
+        shell, mt::fake_shared(mock_ipc_operations), mock_display_selector,
         surface_pixel_formats, report,
         std::make_shared<mtd::NullEventSink>(), resource_cache,
         std::make_shared<mtd::NullScreencast>(),
-         nullptr, nullptr};
+         nullptr, nullptr, nullptr};
 
     mediator.connect(nullptr, &connect_parameters, &connection, null_callback.get());
 
@@ -673,7 +646,7 @@ TEST_F(SessionMediator, fully_packs_buffer_for_create_screencast)
 
     mediator.create_screencast(nullptr, &screencast_parameters,
                                &screencast, null_callback.get());
-    EXPECT_EQ(stub_buffer.id().as_value(), screencast.buffer().buffer_id());
+    EXPECT_EQ(stub_buffer.id().as_value(), screencast.buffer_stream().buffer().buffer_id());
 }
 
 TEST_F(SessionMediator, partially_packs_buffer_for_screencast_buffer)
@@ -734,10 +707,10 @@ TEST_F(SessionMediator, exchange_buffer)
 
     mediator.connect(nullptr, &connect_parameters, &connection, null_callback.get());
     mediator.create_surface(nullptr, &surface_parameters, &surface_response, null_callback.get());
-    EXPECT_THAT(surface_response.buffer().buffer_id(), Eq(stub_buffer1.id().as_value()));
+    EXPECT_THAT(surface_response.buffer_stream().buffer().buffer_id(), Eq(stub_buffer1.id().as_value()));
 
     *buffer_request.mutable_id() = surface_response.id();
-    buffer_request.mutable_buffer()->set_buffer_id(surface_response.buffer().buffer_id());
+    buffer_request.mutable_buffer()->set_buffer_id(surface_response.buffer_stream().buffer().buffer_id());
     mediator.exchange_buffer(nullptr, &buffer_request, &exchanged_buffer, null_callback.get());
     EXPECT_THAT(exchanged_buffer.buffer_id(), Eq(stub_buffer2.id().as_value()));
 }
@@ -762,12 +735,18 @@ TEST_F(SessionMediator, session_exchange_buffer_sends_minimum_information)
     EXPECT_CALL(mock_ipc_operations, pack_buffer(_, Ref(buffer2), mg::BufferIpcMsgType::full_msg))
         .InSequence(seq);
     //swap1
+    EXPECT_CALL(mock_ipc_operations, unpack_buffer(_, Ref(buffer2)))
+        .InSequence(seq);
     EXPECT_CALL(mock_ipc_operations, pack_buffer(_, Ref(buffer1), mg::BufferIpcMsgType::full_msg))
         .InSequence(seq);
     //swap2
+    EXPECT_CALL(mock_ipc_operations, unpack_buffer(_, Ref(buffer1)))
+        .InSequence(seq);
     EXPECT_CALL(mock_ipc_operations, pack_buffer(_, Ref(buffer2), mg::BufferIpcMsgType::update_msg))
         .InSequence(seq);
     //swap3
+    EXPECT_CALL(mock_ipc_operations, unpack_buffer(_, Ref(buffer2)))
+        .InSequence(seq);
     EXPECT_CALL(mock_ipc_operations, pack_buffer(_, Ref(buffer1), mg::BufferIpcMsgType::update_msg))
         .InSequence(seq);
 
@@ -775,7 +754,7 @@ TEST_F(SessionMediator, session_exchange_buffer_sends_minimum_information)
 
     mediator.create_surface(nullptr, &surface_parameters, &surface_response, null_callback.get());
     *buffer_request.mutable_id() = surface_response.id();
-    buffer_request.mutable_buffer()->set_buffer_id(surface_response.buffer().buffer_id());
+    buffer_request.mutable_buffer()->set_buffer_id(surface_response.buffer_stream().buffer().buffer_id());
 
     mediator.exchange_buffer(nullptr, &buffer_request, &exchanged_buffer, null_callback.get());
     buffer_request.mutable_buffer()->set_buffer_id(exchanged_buffer.buffer_id());
@@ -800,7 +779,7 @@ TEST_F(SessionMediator, exchange_buffer_throws_if_client_submits_bad_request)
 
     mediator.connect(nullptr, &connect_parameters, &connection, null_callback.get());
     mediator.create_surface(nullptr, &surface_parameters, &surface_response, null_callback.get());
-    EXPECT_THAT(surface_response.buffer().buffer_id(), Eq(stub_buffer1.id().as_value()));
+    EXPECT_THAT(surface_response.buffer_stream().buffer().buffer_id(), Eq(stub_buffer1.id().as_value()));
 
     *buffer_request.mutable_id() = surface_response.id();
     //client doesnt own stub_buffer2
@@ -817,6 +796,37 @@ TEST_F(SessionMediator, exchange_buffer_throws_if_client_submits_bad_request)
     }, std::logic_error);
 }
 
+TEST_F(SessionMediator, exchange_buffer_different_for_different_surfaces)
+{
+    using namespace testing;
+    mp::SurfaceParameters surface_request;
+    mp::BufferRequest req1;
+    mp::BufferRequest req2;
+    auto const& mock_surface1 = stubbed_session->mock_surface_at(mf::SurfaceId{0});
+    auto const& mock_surface2 = stubbed_session->mock_surface_at(mf::SurfaceId{1});
+    Sequence seq;
+    EXPECT_CALL(*mock_surface1, swap_buffers(_,_))
+        .InSequence(seq);
+    EXPECT_CALL(*mock_surface2, swap_buffers(_,_))
+        .InSequence(seq);
+    EXPECT_CALL(*mock_surface2, swap_buffers(_,_))
+        .InSequence(seq);
+    EXPECT_CALL(*mock_surface1, swap_buffers(_,_))
+        .InSequence(seq);
+
+    mediator.connect(nullptr, &connect_parameters, &connection, null_callback.get());
+
+    mediator.create_surface(nullptr, &surface_request, &surface_response, null_callback.get());
+    *req1.mutable_id() = surface_response.id();
+    *req1.mutable_buffer() = surface_response.buffer_stream().buffer();
+    mediator.create_surface(nullptr, &surface_request, &surface_response, null_callback.get());
+    *req2.mutable_id() = surface_response.id();
+    *req2.mutable_buffer() = surface_response.buffer_stream().buffer();
+    mediator.exchange_buffer(nullptr, &req2, &buffer_response, null_callback.get());
+    mediator.exchange_buffer(nullptr, &req1, &buffer_response, null_callback.get());
+    mediator.disconnect(nullptr, nullptr, nullptr, null_callback.get());
+}
+
 TEST_F(SessionMediator, buffer_fd_resources_are_put_in_resource_cache)
 {
     using namespace testing;
@@ -840,15 +850,15 @@ TEST_F(SessionMediator, buffer_fd_resources_are_put_in_resource_cache)
     EXPECT_CALL(mock_cache, save_fd(_,fake_fd2));
 
     mf::SessionMediator mediator{
-        shell, graphics_platform, graphics_changer,
+        shell, mt::fake_shared(mock_ipc_operations), graphics_changer,
         surface_pixel_formats, report,
         std::make_shared<mtd::NullEventSink>(),
-        mt::fake_shared(mock_cache), stub_screencast, nullptr, nullptr};
+        mt::fake_shared(mock_cache), stub_screencast, &connector, nullptr, nullptr};
 
     mediator.connect(nullptr, &connect_parameters, &connection, null_callback.get());
     mediator.create_surface(nullptr, &surface_parameters, &surface_response, null_callback.get());
     *buffer_request.mutable_id() = surface_response.id();
-    buffer_request.mutable_buffer()->set_buffer_id(surface_response.buffer().buffer_id());
+    buffer_request.mutable_buffer()->set_buffer_id(surface_response.buffer_stream().buffer().buffer_id());
 
     mediator.exchange_buffer(nullptr, &buffer_request, &exchanged_buffer, null_callback.get());
     buffer_request.mutable_buffer()->set_buffer_id(exchanged_buffer.buffer_id());
@@ -858,3 +868,30 @@ TEST_F(SessionMediator, buffer_fd_resources_are_put_in_resource_cache)
     buffer_request.mutable_buffer()->set_buffer_id(exchanged_buffer.buffer_id());
     buffer_request.mutable_buffer()->clear_fd();
 }
+
+//FIXME: we have an platform specific request in the protocol!
+TEST_F(SessionMediator, drm_auth_magic_calls_platform_operation_abstraction)
+{
+    using namespace testing;
+
+    int magic{0x3248};
+    int test_response{4};
+    mg::PlatformOperationMessage response;
+    response.data.resize(sizeof(int));
+    *(reinterpret_cast<int*>(response.data.data())) = test_response;
+
+    mg::PlatformOperationMessage request;
+    drm_request.set_magic(magic);
+
+    EXPECT_CALL(mock_ipc_operations, platform_operation(_, _))
+        .Times(1)
+        .WillOnce(DoAll(SaveArg<1>(&request), Return(response)));
+
+    mediator.connect(nullptr, &connect_parameters, &connection, null_callback.get());
+    mediator.drm_auth_magic(nullptr, &drm_request, &drm_response, null_callback.get());
+    mediator.disconnect(nullptr, nullptr, nullptr, null_callback.get());
+
+    ASSERT_THAT(request.data.size(), Eq(sizeof(int)));
+    EXPECT_THAT(*(reinterpret_cast<int*>(request.data.data())), Eq(magic));
+    EXPECT_THAT(drm_response.status_code(), Eq(test_response));
+}
