@@ -16,7 +16,8 @@
  * Authored by: Kevin DuBois <kevin.dubois@canonical.com>
  */
 
-#include "src/platform/graphics/android/resource_factory.h"
+#include "src/platforms/android/server/resource_factory.h"
+#include "src/platforms/android/server/hwc_loggers.h"
 #include "mir_test_doubles/mock_android_hw.h"
 
 #include <stdexcept>
@@ -30,6 +31,7 @@ namespace mtd=mir::test::doubles;
 struct ResourceFactoryTest  : public ::testing::Test
 {
     mtd::HardwareAccessMock hw_access_mock;
+    std::shared_ptr<mga::HwcReport> null_report{std::make_shared<mga::NullHwcReport>()};
 };
 
 TEST_F(ResourceFactoryTest, fb_native_creation_opens_and_closes_gralloc)
@@ -70,11 +72,14 @@ TEST_F(ResourceFactoryTest, test_device_creation_throws_on_failure)
 TEST_F(ResourceFactoryTest, hwc_allocation)
 {
     using namespace testing;
+    hw_access_mock.mock_hwc_device->common.version = HWC_DEVICE_API_VERSION_1_2;
     EXPECT_CALL(hw_access_mock, hw_get_module(StrEq(HWC_HARDWARE_MODULE_ID), _))
         .Times(1);
 
     mga::ResourceFactory factory;
-    factory.create_hwc_native_device();
+    auto wrapper_tuple = factory.create_hwc_wrapper(null_report);
+    EXPECT_THAT(std::get<1>(wrapper_tuple), Eq(mga::HwcVersion::hwc12));
+    std::get<0>(wrapper_tuple).reset();
 
     EXPECT_TRUE(hw_access_mock.open_count_matches_close());
 }
@@ -93,10 +98,10 @@ TEST_F(ResourceFactoryTest, hwc_allocation_failures)
     mga::ResourceFactory factory;
 
     EXPECT_THROW({
-        factory.create_hwc_native_device();
+        factory.create_hwc_wrapper(null_report);
     }, std::runtime_error);
     EXPECT_THROW({
-        factory.create_hwc_native_device();
+        factory.create_hwc_wrapper(null_report);
     }, std::runtime_error);
 
     EXPECT_TRUE(hw_access_mock.open_count_matches_close());
