@@ -19,14 +19,14 @@
 #include "mir/graphics/display.h"
 #include "mir/graphics/display_buffer.h"
 #include "mir/graphics/display_configuration.h"
-#include "src/platform/graphics/mesa/platform.h"
-#include "src/server/report/null_report_factory.h"
+#include "mir/graphics/platform.h"
 
 #include "mir_test_doubles/mock_egl.h"
 #include "mir_test_doubles/mock_gl.h"
 #include "mir/graphics/display_configuration_policy.h"
-#include "mir_test_doubles/null_virtual_terminal.h"
 #include "mir_test_doubles/stub_gl_config.h"
+#include "mir_test_doubles/stub_gl_program_factory.h"
+#include "mir_test_doubles/platform_factory.h"
 
 #include "mir_test_framework/udev_environment.h"
 
@@ -39,11 +39,9 @@
 #include <unordered_set>
 
 namespace mg = mir::graphics;
-namespace mgm = mir::graphics::mesa;
 namespace geom = mir::geometry;
 namespace mtd = mir::test::doubles;
-namespace mtf = mir::mir_test_framework;
-namespace mr = mir::report;
+namespace mtf = mir_test_framework;
 
 namespace
 {
@@ -115,13 +113,8 @@ public:
                                  SetArgPointee<4>(1),
                                  Return(EGL_TRUE)));
 
-        const char* egl_exts = "EGL_KHR_image EGL_KHR_image_base EGL_MESA_drm_image";
-        const char* gl_exts = "GL_OES_texture_npot GL_OES_EGL_image";
-
-        ON_CALL(mock_egl, eglQueryString(_,EGL_EXTENSIONS))
-            .WillByDefault(Return(egl_exts));
-        ON_CALL(mock_gl, glGetString(GL_EXTENSIONS))
-            .WillByDefault(Return(reinterpret_cast<const GLubyte*>(gl_exts)));
+        mock_egl.provide_egl_extensions();
+        mock_gl.provide_gles_extensions();
 
         /*
          * Silence uninteresting calls called when cleaning up resources in
@@ -135,11 +128,9 @@ public:
         fake_devices.add_standard_device("standard-drm-devices");
     }
 
-    std::shared_ptr<mgm::Platform> create_platform()
+    std::shared_ptr<mg::Platform> create_platform()
     {
-        return std::make_shared<mgm::Platform>(
-            mr::null_display_report(),
-            std::make_shared<mtd::NullVirtualTerminal>());
+        return mtd::create_platform_with_null_dependencies();
     }
 
     std::shared_ptr<mg::Display> create_display_cloned(
@@ -147,6 +138,7 @@ public:
     {
         return platform->create_display(
             std::make_shared<ClonedDisplayConfigurationPolicy>(),
+            std::make_shared<mtd::StubGLProgramFactory>(),
             std::make_shared<mtd::StubGLConfig>());
     }
 
@@ -155,6 +147,7 @@ public:
     {
         return platform->create_display(
             std::make_shared<SideBySideDisplayConfigurationPolicy>(),
+            std::make_shared<mtd::StubGLProgramFactory>(),
             std::make_shared<mtd::StubGLConfig>());
     }
 
@@ -242,8 +235,8 @@ TEST_F(MesaDisplayMultiMonitorTest, create_display_sets_all_connected_crtcs)
     setup_outputs(num_connected_outputs, num_disconnected_outputs);
 
     /* Create DRM FBs */
-    EXPECT_CALL(mock_drm, drmModeAddFB(mock_drm.fake_drm.fd(),
-                                       _, _, _, _, _, _, _))
+    EXPECT_CALL(mock_drm, drmModeAddFB2(mock_drm.fake_drm.fd(),
+                                        _, _, _, _, _, _, _, _))
         .WillRepeatedly(DoAll(SetArgPointee<7>(fb_id), Return(0)));
 
     ExpectationSet crtc_setups;
@@ -322,7 +315,7 @@ ACTION_P(InvokePageFlipHandler, param)
 
 }
 
-TEST_F(MesaDisplayMultiMonitorTest, post_update_flips_all_connected_crtcs)
+TEST_F(MesaDisplayMultiMonitorTest, flip_flips_all_connected_crtcs)
 {
     using namespace testing;
 
@@ -334,8 +327,8 @@ TEST_F(MesaDisplayMultiMonitorTest, post_update_flips_all_connected_crtcs)
     setup_outputs(num_connected_outputs, num_disconnected_outputs);
 
     /* Create DRM FBs */
-    EXPECT_CALL(mock_drm, drmModeAddFB(mock_drm.fake_drm.fd(),
-                                       _, _, _, _, _, _, _))
+    EXPECT_CALL(mock_drm, drmModeAddFB2(mock_drm.fake_drm.fd(),
+                                        _, _, _, _, _, _, _, _))
         .WillRepeatedly(DoAll(SetArgPointee<7>(fb_id), Return(0)));
 
     /* All crtcs are flipped */
@@ -363,14 +356,16 @@ TEST_F(MesaDisplayMultiMonitorTest, post_update_flips_all_connected_crtcs)
     /* First frame: Page flips are scheduled, but not waited for */
     display->for_each_display_buffer([](mg::DisplayBuffer& buffer)
     {
-        buffer.post_update();
+        buffer.gl_swap_buffers();
+        buffer.flip();
     });
 
     /* Second frame: Previous page flips finish (drmHandleEvent) and new ones
        are scheduled */
     display->for_each_display_buffer([](mg::DisplayBuffer& buffer)
     {
-        buffer.post_update();
+        buffer.gl_swap_buffers();
+        buffer.flip();
     });
 }
 
@@ -381,9 +376,9 @@ struct FBIDContainer
 {
     FBIDContainer(uint32_t base_fb_id) : last_fb_id{base_fb_id} {}
 
-    int add_fb(int, uint32_t, uint32_t, uint8_t,
-               uint8_t, uint32_t, uint32_t,
-               uint32_t *buf_id)
+    int add_fb2(int, uint32_t, uint32_t, uint32_t,
+               uint32_t[4], uint32_t[4], uint32_t[4],
+               uint32_t *buf_id, uint32_t)
     {
         *buf_id = last_fb_id;
         fb_ids.insert(last_fb_id);
@@ -422,10 +417,10 @@ TEST_F(MesaDisplayMultiMonitorTest, create_display_uses_different_drm_fbs_for_si
     setup_outputs(num_connected_outputs, num_disconnected_outputs);
 
     /* Create DRM FBs */
-    EXPECT_CALL(mock_drm, drmModeAddFB(mock_drm.fake_drm.fd(),
-                                       _, _, _, _, _, _, _))
+    EXPECT_CALL(mock_drm, drmModeAddFB2(mock_drm.fake_drm.fd(),
+                                        _, _, _, _, _, _, _, _))
         .Times(num_connected_outputs)
-        .WillRepeatedly(Invoke(&fb_id_container, &FBIDContainer::add_fb));
+        .WillRepeatedly(Invoke(&fb_id_container, &FBIDContainer::add_fb2));
 
     ExpectationSet crtc_setups;
 
