@@ -16,15 +16,17 @@
  * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
-#include "src/platform/graphics/mesa/linux_virtual_terminal.h"
+#include "src/platforms/mesa/server/linux_virtual_terminal.h"
 #include "src/server/report/null_report_factory.h"
-#include "mir/main_loop.h"
+#include "mir/graphics/event_handler_register.h"
 
 #include "mir_test/fake_shared.h"
 #include "mir_test_doubles/mock_display_report.h"
+#include "mir_test/gmock_fixes.h"
 
 #include <gtest/gtest.h>
 #include <gmock/gmock.h>
+#include "mir_test/gmock_fixes.h"
 
 #include <stdexcept>
 
@@ -68,21 +70,21 @@ public:
 // Add a typedef to aid clarity.
 typedef testing::NiceMock<MockPosixProcessOperations> StubPosixProcessOperations;
 
-class MockMainLoop : public mir::MainLoop
+class MockEventHandlerRegister : public mg::EventHandlerRegister
 {
 public:
-    ~MockMainLoop() noexcept {}
-
-    void run() {}
-    void stop() {}
+    ~MockEventHandlerRegister() noexcept {}
 
     MOCK_METHOD2(register_signal_handler,
                  void(std::initializer_list<int>,
                       std::function<void(int)> const&));
 
-    MOCK_METHOD2(register_fd_handler,
+    MOCK_METHOD3(register_fd_handler,
                  void(std::initializer_list<int>,
+                      void const*,
                       std::function<void(int)> const&));
+
+    MOCK_METHOD1(unregister_fd_handler, void(void const*));
 };
 
 ACTION_TEMPLATE(SetIoctlPointee,
@@ -202,7 +204,7 @@ public:
     {
         using namespace testing;
 
-        EXPECT_CALL(mock_main_loop, register_signal_handler(ElementsAre(sig), _))
+        EXPECT_CALL(mock_event_handler_register, register_signal_handler(ElementsAre(sig), _))
             .WillOnce(SaveArg<1>(&sig_handler));
         EXPECT_CALL(mock_fops, ioctl(fake_vt_fd, VT_SETMODE,
                                      MatcherCast<void*>(ModeUsesSignal(sig))));
@@ -217,6 +219,16 @@ public:
     {
         using namespace testing;
 
+        set_up_expectations_for_vt_restore(vt_mode);
+
+        EXPECT_CALL(mock_fops, close(fake_vt_fd))
+            .WillOnce(Return(0));
+    }
+
+    void set_up_expectations_for_vt_restore(vt_mode const& vt_mode)
+    {
+        using namespace testing;
+
         EXPECT_CALL(mock_fops, tcsetattr(fake_vt_fd, TCSANOW, An<const struct termios *>()))
             .WillOnce(Return(0));
         EXPECT_CALL(mock_fops, ioctl(fake_vt_fd, KDSKBMODE, fake_kb_mode))
@@ -235,9 +247,6 @@ public:
             EXPECT_CALL(mock_fops, ioctl(fake_vt_fd, VT_SETMODE, An<void*>()))
                 .Times(0);
         }
-
-        EXPECT_CALL(mock_fops, close(fake_vt_fd))
-            .WillOnce(Return(0));
     }
 
     int const fake_vt_fd;
@@ -248,7 +257,7 @@ public:
     struct termios fake_tc_attr;
     std::function<void(int)> sig_handler;
     MockVTFileOperations mock_fops;
-    MockMainLoop mock_main_loop;
+    MockEventHandlerRegister mock_event_handler_register;
 };
 
 
@@ -406,7 +415,7 @@ TEST_F(LinuxVirtualTerminalTest, uses_sigusr1_for_switch_handling)
     mgm::LinuxVirtualTerminal vt(fops, std::move(pops), 0, null_report);
 
     auto null_handler = [] { return true; };
-    vt.register_switch_handlers(mock_main_loop, null_handler, null_handler);
+    vt.register_switch_handlers(mock_event_handler_register, null_handler, null_handler);
 }
 
 TEST_F(LinuxVirtualTerminalTest, allows_vt_switch_on_switch_away_handler_success)
@@ -433,7 +442,7 @@ TEST_F(LinuxVirtualTerminalTest, allows_vt_switch_on_switch_away_handler_success
     mgm::LinuxVirtualTerminal vt(fops, std::move(pops), 0, null_report);
 
     auto succeeding_handler = [] { return true; };
-    vt.register_switch_handlers(mock_main_loop, succeeding_handler, succeeding_handler);
+    vt.register_switch_handlers(mock_event_handler_register, succeeding_handler, succeeding_handler);
 
     /* Fake a VT switch away request */
     sig_handler(SIGUSR1);
@@ -471,7 +480,7 @@ TEST_F(LinuxVirtualTerminalTest, disallows_vt_switch_on_switch_away_handler_fail
     mgm::LinuxVirtualTerminal vt(fops, std::move(pops), 0, mt::fake_shared(mock_report));
 
     auto failing_handler = [] { return false; };
-    vt.register_switch_handlers(mock_main_loop, failing_handler, failing_handler);
+    vt.register_switch_handlers(mock_event_handler_register, failing_handler, failing_handler);
 
     /* Fake a VT switch away request */
     sig_handler(SIGUSR1);
@@ -509,7 +518,7 @@ TEST_F(LinuxVirtualTerminalTest, reports_failed_vt_switch_back_attempt)
 
     auto succeeding_handler = [] { return true; };
     auto failing_handler = [] { return false; };
-    vt.register_switch_handlers(mock_main_loop, succeeding_handler, failing_handler);
+    vt.register_switch_handlers(mock_event_handler_register, succeeding_handler, failing_handler);
 
     /* Fake a VT switch away request */
     sig_handler(SIGUSR1);
@@ -655,3 +664,28 @@ TEST_F(LinuxVirtualTerminalTest, exception_if_becoming_session_leader_fails)
         mgm::LinuxVirtualTerminal vt(fops, std::move(pops), vt_num, null_report);
     }, std::runtime_error);
 }
+
+TEST_F(LinuxVirtualTerminalTest, restores_keyboard_and_graphics)
+{
+    using namespace testing;
+
+    int const vt_num{7};
+
+    InSequence s;
+
+    set_up_expectations_for_vt_setup(vt_num, true);
+
+    set_up_expectations_for_vt_restore(fake_vt_mode_auto);
+
+    auto fops = mt::fake_shared<mgm::VTFileOperations>(mock_fops);
+    auto pops = std::unique_ptr<mgm::PosixProcessOperations>(new StubPosixProcessOperations());
+    auto null_report = mr::null_display_report();
+
+    mgm::LinuxVirtualTerminal vt(fops, std::move(pops), vt_num, null_report);
+
+    vt.restore();
+
+    Mock::VerifyAndClearExpectations(&mock_fops);
+
+    set_up_expectations_for_vt_teardown();
+}
