@@ -20,11 +20,11 @@
 #include "src/server/input/android/android_input_thread.h"
 #include "src/server/input/android/android_input_constants.h"
 #include "src/server/input/android/android_input_channel.h"
+#include "src/server/input/android/input_channel_factory.h"
 
 #include "mir/input/input_channel.h"
 
 #include "mir_test/fake_shared.h"
-#include "mir_test_doubles/mock_input_dispatcher.h"
 #include "mir_test_doubles/stub_input_channel.h"
 
 #include <EventHub.h>
@@ -69,7 +69,7 @@ struct MockEventHub : public droidinput::EventHubInterface
     MOCK_CONST_METHOD2(getVirtualKeyDefinitions, void(int32_t, droidinput::Vector<droidinput::VirtualKeyDefinition>&));
     MOCK_CONST_METHOD1(getKeyCharacterMap, droidinput::sp<droidinput::KeyCharacterMap>(int32_t));
     MOCK_METHOD2(setKeyboardLayoutOverlay, bool(int32_t, const droidinput::sp<droidinput::KeyCharacterMap>&));
-    MOCK_METHOD2(vibrate, void(int32_t, nsecs_t));
+    MOCK_METHOD2(vibrate, void(int32_t, std::chrono::nanoseconds));
     MOCK_METHOD1(cancelVibrate, void(int32_t));
     MOCK_METHOD0(requestReopenDevices, void());
     MOCK_METHOD0(wake, void());
@@ -93,68 +93,64 @@ namespace
 
 struct AndroidInputManagerSetup : public testing::Test
 {
-    AndroidInputManagerSetup()
-    {
-        using namespace ::testing;
+    std::shared_ptr<MockEventHub> event_hub = std::make_shared<MockEventHub>();
+    std::shared_ptr<MockInputThread> reader_thread = std::make_shared<MockInputThread>();
 
-        event_hub = new MockEventHub();
-        dispatcher = new mtd::MockInputDispatcher();
-        dispatcher_thread = std::make_shared<MockInputThread>();
-        reader_thread = std::make_shared<MockInputThread>();
+    void setup_start_expectations()
+    {
+        testing::InSequence seq;
+
+        EXPECT_CALL(*event_hub, flush());
+        EXPECT_CALL(*reader_thread, start());
     }
 
-    droidinput::sp<MockEventHub> event_hub;
-    droidinput::sp<mtd::MockInputDispatcher> dispatcher;
-    std::shared_ptr<MockInputThread> dispatcher_thread;
-    std::shared_ptr<MockInputThread> reader_thread;
-};
-
-}
-
-TEST_F(AndroidInputManagerSetup, start_and_stop)
-{
-    using namespace ::testing;
-
-    ExpectationSet dispatcher_setup;
-    ExpectationSet reader_setup;
-
-    dispatcher_setup += EXPECT_CALL(*dispatcher,
-                                    setInputDispatchMode(mia::DispatchEnabled,
-                                                         mia::DispatchUnfrozen))
-                            .Times(1);
-    dispatcher_setup += EXPECT_CALL(*dispatcher, setInputFilterEnabled(true)).Times(1);
-
-    reader_setup += EXPECT_CALL(*event_hub, flush()).Times(1);
-
-    EXPECT_CALL(*reader_thread, start())
-        .Times(1)
-        .After(reader_setup);
-
-    EXPECT_CALL(*dispatcher_thread, start())
-        .Times(1)
-        .After(dispatcher_setup);
-
+    void setup_stop_expectations()
     {
-        InSequence seq;
+        testing::InSequence seq;
 
-        EXPECT_CALL(*dispatcher_thread, request_stop());
-        EXPECT_CALL(*dispatcher, setInputDispatchMode(mia::DispatchDisabled, mia::DispatchFrozen)).Times(1);
-        EXPECT_CALL(*dispatcher_thread, join());
         EXPECT_CALL(*reader_thread, request_stop());
         EXPECT_CALL(*event_hub, wake());
         EXPECT_CALL(*reader_thread, join());
     }
+};
 
-    mia::InputManager manager(event_hub, dispatcher, reader_thread, dispatcher_thread);
+}
+
+TEST_F(AndroidInputManagerSetup, starts_and_stops_reader)
+{
+    using namespace ::testing;
+
+    setup_start_expectations();
+    setup_stop_expectations();
+
+    mia::InputManager manager(event_hub, reader_thread);
 
     manager.start();
     manager.stop();
+
+    // The input manager is unconditionally stopped at destruction.
+    Mock::VerifyAndClearExpectations(reader_thread.get());
+    Mock::VerifyAndClearExpectations(event_hub.get());
+
+    setup_stop_expectations();
 }
 
-TEST_F(AndroidInputManagerSetup, manager_returns_input_channel_with_fds)
+TEST_F(AndroidInputManagerSetup, stops_reader_at_destruction)
 {
-    mia::InputManager manager(event_hub, dispatcher, reader_thread, dispatcher_thread);
+    using namespace ::testing;
 
-    auto package = manager.make_input_channel();
+    setup_start_expectations();
+    setup_stop_expectations();
+
+    mia::InputManager manager(event_hub, reader_thread);
+
+    manager.start();
+}
+
+TEST_F(AndroidInputManagerSetup, channel_factory_returns_input_channel_with_fds)
+{
+    mia::InputChannelFactory factory;
+
+    auto package = factory.make_input_channel();
     EXPECT_NE(nullptr, std::dynamic_pointer_cast<mia::AndroidInputChannel>(package));
 }
