@@ -20,8 +20,9 @@
 
 #include "src/server/input/android/android_input_registrar.h"
 
-#include "mir_test_doubles/mock_input_dispatcher.h"
-#include "mir_test_doubles/stub_input_channel.h"
+#include "mir_test_doubles/mock_android_input_dispatcher.h"
+#include "mir_test_doubles/stub_scene_surface.h"
+#include "mir_test_doubles/stub_scene.h"
 
 #include "mir_test/fake_shared.h"
 
@@ -35,10 +36,8 @@
 
 namespace mi = mir::input;
 namespace mia = mi::android;
-namespace ms = mir::scene;
 namespace mt = mir::test;
 namespace mtd = mt::doubles;
-namespace geom = mir::geometry;
 
 namespace
 {
@@ -48,38 +47,19 @@ namespace
 // valid fds to allow non-throwing construction of a real input channel.
 struct AndroidInputRegistrarFdSetup : public testing::Test
 {
-    void SetUp() override
+    AndroidInputRegistrarFdSetup()
+        : surface(socket(AF_UNIX, SOCK_SEQPACKET, 0))
     {
-        test_input_fd = socket(AF_UNIX, SOCK_SEQPACKET, 0);
-
-        dispatcher = new mtd::MockInputDispatcher();
+        registrar.set_dispatcher(dispatcher);
     }
-    void TearDown() override
+    ~AndroidInputRegistrarFdSetup()
     {
-        close(test_input_fd);
-    }
-    int test_input_fd;
-    droidinput::sp<mtd::MockInputDispatcher> dispatcher;
-};
-
-struct StubInputSurface : public mi::Surface
-{
-    geom::Point top_left() const
-    {
-        return geom::Point();
-    }
-    geom::Size size() const
-    {
-        return geom::Size();
-    }
-    std::string name() const
-    {
-        return {};
-    }
-    bool contains(geom::Point const&) const
-    {
-        return true;
+        close(surface.fd);
     }
+    std::shared_ptr<mtd::MockAndroidInputDispatcher> dispatcher = std::make_shared<mtd::MockAndroidInputDispatcher>();
+    std::shared_ptr<mtd::StubScene> scene = std::make_shared<mtd::StubScene>();
+    mtd::StubSceneSurface surface;
+    mia::InputRegistrar registrar{scene};
 };
 
 MATCHER_P(WindowHandleFor, channel, "")
@@ -95,18 +75,13 @@ TEST_F(AndroidInputRegistrarFdSetup, input_channel_opened_behavior)
 {
     using namespace ::testing;
 
-    auto channel = std::make_shared<mtd::StubInputChannel>(test_input_fd);
-    auto surface = std::make_shared<StubInputSurface>();
-
-    EXPECT_CALL(*dispatcher, registerInputChannel(_, WindowHandleFor(channel), _)).Times(1)
+    EXPECT_CALL(*dispatcher, registerInputChannel(_, WindowHandleFor(surface.input_channel()), _)).Times(1)
         .WillOnce(Return(droidinput::OK));
 
-    mia::InputRegistrar registrar(dispatcher);
-
-    registrar.input_channel_opened(channel, surface, mi::InputReceptionMode::normal);
+    registrar.add_window_handle_for_surface(&surface);
     EXPECT_THROW({
             // We can't open a surface twice
-            registrar.input_channel_opened(channel, surface, mi::InputReceptionMode::normal);
+            registrar.add_window_handle_for_surface(&surface);
     }, std::logic_error);
 }
 
@@ -114,23 +89,19 @@ TEST_F(AndroidInputRegistrarFdSetup, input_channel_closed_behavior)
 {
     using namespace ::testing;
 
-    auto channel = std::make_shared<mtd::StubInputChannel>(test_input_fd);
-    auto surface = std::make_shared<StubInputSurface>();
-
-    EXPECT_CALL(*dispatcher, registerInputChannel(_, WindowHandleFor(channel), _)).Times(1)
+    EXPECT_CALL(*dispatcher, registerInputChannel(_, WindowHandleFor(surface.input_channel()), _)).Times(1)
         .WillOnce(Return(droidinput::OK));
     EXPECT_CALL(*dispatcher, unregisterInputChannel(_)).Times(1);
-    mia::InputRegistrar registrar(dispatcher);
 
     EXPECT_THROW({
             // We can't close a surface which hasn't been opened
-            registrar.input_channel_closed(channel);
+            registrar.remove_window_handle_for_surface(&surface);
     }, std::logic_error);
-    registrar.input_channel_opened(channel, surface, mi::InputReceptionMode::normal);
-    registrar.input_channel_closed(channel);
+    registrar.add_window_handle_for_surface(&surface);
+    registrar.remove_window_handle_for_surface(&surface);
     EXPECT_THROW({
             // Nor can we close a surface twice
-            registrar.input_channel_closed(channel);
+            registrar.remove_window_handle_for_surface(&surface);
     }, std::logic_error);
 }
 
@@ -138,13 +109,10 @@ TEST_F(AndroidInputRegistrarFdSetup, monitor_flag_is_passed_to_dispatcher)
 {
     using namespace ::testing;
 
-    auto channel = std::make_shared<mtd::StubInputChannel>(test_input_fd);
-    auto surface = std::make_shared<StubInputSurface>();
+    surface.set_reception_mode(mi::InputReceptionMode::receives_all_input);
 
     EXPECT_CALL(*dispatcher, registerInputChannel(_, _, true)).Times(1)
         .WillOnce(Return(droidinput::OK));
 
-    mia::InputRegistrar registrar(dispatcher);
-
-    registrar.input_channel_opened(channel, surface, mi::InputReceptionMode::receives_all_input);
+    registrar.add_window_handle_for_surface(&surface);
 }
