@@ -19,11 +19,14 @@
 #include "src/server/input/android/android_input_target_enumerator.h"
 
 #include "mir/input/input_channel.h"
-#include "mir/input/input_targets.h"
+#include "mir/input/scene.h"
+#include "mir/input/surface.h"
 
 #include "mir_test/fake_shared.h"
 #include "mir_test_doubles/stub_input_channel.h"
 #include "mir_test_doubles/stub_input_handles.h"
+#include "mir_test_doubles/stub_input_surface.h"
+#include "mir_test_doubles/stub_input_scene.h"
 #include "mir_test_doubles/mock_window_handle_repository.h"
 
 #include <InputDispatcher.h>
@@ -37,26 +40,37 @@
 
 namespace mi = mir::input;
 namespace mia = mir::input::android;
+namespace ms = mir::scene;
+namespace geom = mir::geometry;
+
 namespace mt = mir::test;
 namespace mtd = mir::test::doubles;
 
 namespace
 {
 
-struct StubInputTargets : public mi::InputTargets
+struct StubScene : public mtd::StubInputScene
 {
-    StubInputTargets(std::initializer_list<std::shared_ptr<mi::InputChannel>> const& target_list)
+    StubScene(std::initializer_list<std::shared_ptr<mi::Surface>> const& target_list)
         : targets(target_list.begin(), target_list.end())
     {
     }
 
-    void for_each(std::function<void(std::shared_ptr<mi::InputChannel> const&)> const& callback) override
+    void for_each(std::function<void(std::shared_ptr<mi::Surface> const&)> const& callback) override
     {
         for (auto target : targets)
             callback(target);
     }
 
-    std::vector<std::shared_ptr<mi::InputChannel>> targets;
+    void add_observer(std::shared_ptr<ms::Observer> const& /* observer */)
+    {
+    }
+
+    void remove_observer(std::weak_ptr<ms::Observer> const& /* observer */)
+    {
+    }
+
+    std::vector<std::shared_ptr<mi::Surface>> targets;
 };
 
 }
@@ -64,21 +78,23 @@ struct StubInputTargets : public mi::InputTargets
 TEST(AndroidInputTargetEnumerator, enumerates_registered_handles_for_surfaces)
 {
     using namespace ::testing;
-    std::shared_ptr<mi::InputChannel> t1, t2;
-    t1 = std::make_shared<mtd::StubInputChannel>();
-    t2 = std::make_shared<mtd::StubInputChannel>();
+
+    std::shared_ptr<mi::InputChannel> channel1 = std::make_shared<mtd::StubInputChannel>();
+    std::shared_ptr<mi::InputChannel> channel2 = std::make_shared<mtd::StubInputChannel>();
+    auto target1 = std::make_shared<mtd::StubInputSurface>(channel1);
+    auto target2 = std::make_shared<mtd::StubInputSurface>(channel2);
     mtd::MockWindowHandleRepository repository;
     droidinput::sp<droidinput::InputWindowHandle> stub_window_handle1 = new mtd::StubWindowHandle;
     droidinput::sp<droidinput::InputWindowHandle> stub_window_handle2 = new mtd::StubWindowHandle;
-    StubInputTargets targets({t1, t2});
+    StubScene targets({target1, target2});
 
     Sequence seq2;
     EXPECT_CALL(repository, handle_for_channel(
-        std::const_pointer_cast<mi::InputChannel const>(t1)))
+        std::const_pointer_cast<mi::InputChannel const>(channel1)))
         .InSequence(seq2)
         .WillOnce(Return(stub_window_handle1));
     EXPECT_CALL(repository, handle_for_channel(
-        std::const_pointer_cast<mi::InputChannel const>(t2)))
+        std::const_pointer_cast<mi::InputChannel const>(channel2)))
         .InSequence(seq2)
         .WillOnce(Return(stub_window_handle2));
 
